/*
 *  Copyright (C) 1999-2004 Anoto Group AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

// TODO: Check for adding irq locks

#include <linux/module.h>
#include <asm/system.h>
#ifdef CONFIG_ARGUS_3
#include <asm/arch/argus3.h>
#else
#include <asm/arch/argus4.h>
#endif
#include <asm/arch/argdma.h>

#define ASSERT(x) printk("Asserted...\n")

#ifdef CONFIG_ARGUS_3
#define CHANNEL_CNT (10)
#define DMA_CHANNEL(cha) (&SOC_DMA.CHANNEL[cha])

// MDEV data address for each port. 0 means FDEV or not available.
static const unsigned int MDEVADDR[] =
{
  0,                  // DMA_PORT_AVP_BAYER_OUT 0
  0,                  // DMA_PORT_AVP_Y         1
  0,                  // DMA_PORT_AVP_U         2
  0,                  // DMA_PORT_AVP_V         3
  0,                  // DMA_PORT_AVP_ANALYZE   4
  0,                  // DMA_PORT_AVP_BAYER_IN  5
  0,                  // DMA_PORT_ETHRX         6
  0,                  // DMA_PORT_ETHTX         7
  0,                  // N/A                    8
  0,                  // N/A                    9
  0,                  // N/A                   10
  0,                  // N/A                   11
  0,                  // N/A                   12
  0,                  // N/A                   13
  0,                  // N/A                   14
  0,                  // N/A                   15
  0x200d0040,         // DMA_PORT_USB_EP2_RD   16
  0x200d0044,         // DMA_PORT_USB_EP2_WR   17
  0x200d0048,         // DMA_PORT_USB_EP3_RD   18
  0x200d004c,         // DMA_PORT_USB_EP3_WR   19
  0x200b0000,         // DMA_PORT_UART0_RD     20
  0x200b0000,         // DMA_PORT_UART0_WR     21
  0x200c0000,         // DMA_PORT_UART1_RD     22
  0x200c0000,         // DMA_PORT_UART1_WR     23
  0x200f0008,         // DMA_PORT_SSI_IN       24
  0x200f0008,         // DMA_PORT_SSI_OUT      25
  0,                  // N/A                   26
  0,                  // N/A                   27
  0,                  // N/A                   28
  0,                  // N/A                   29
  0,                  // N/A                   30
  0                   // DMA_PORT_ALWAYS       31
};
#else
#define CHANNEL_CNT (20)
#define DMA_CHANNEL(cha) (&SOC_DMA[(cha)&1].CHANNEL[(cha)>>1])

// MDEV data address for each port. 0 means FDEV or not available.
static const unsigned int MDEVADDR[] =
{
  0,                  // DMA_PORT_SMIA          0
  0,                  // DMA_PORT_AVP_BAYER_OUT 1
  0,                  // DMA_PORT_AVP_Y0        2
  0,                  // DMA_PORT_AVP_C0        3
  0,                  // DMA_PORT_AVP_Y1        4
  0,                  // DMA_PORT_AVP_C1        5
  0,                  // DMA_PORT_AVP_ANALYZE   6
  0,                  // DMA_PORT_AVP_BAYER_IN  7
  0,                  // DMA_PORT_VIDOUT_Y      8
  0,                  // DMA_PORT_VIDOUT_UV     9
  0,                  // DMA_PORT_VIDOUT_RGBA   10
  0,                  // DMA_PORT_JPEG_OUT      11
  0,                  // DMA_PORT_JPEG_Y        12
  0,                  // DMA_PORT_JPEG_UV       13
  0x20100020,         // DMA_PORT_SDHC0         14
  0x20110020,         // DMA_PORT_SDHC1         15
  0,                  // DMA_PORT_AES_IN        16
  0,                  // DMA_PORT_AES_OUT       17
  0x200d0040,         // DMA_PORT_USB_EP2_RD    18
  0x200d0044,         // DMA_PORT_USB_EP2_WR    19
  0x200d0048,         // DMA_PORT_USB_EP3_RD    20
  0x200d004c,         // DMA_PORT_USB_EP3_WR    21
  0x200b0000,         // DMA_PORT_UART0_RD      22
  0x200b0000,         // DMA_PORT_UART0_WR      23
  0x200c0000,         // DMA_PORT_UART1_RD      24
  0x200c0000,         // DMA_PORT_UART1_WR      25
  0x200f0008,         // DMA_PORT_SSI_IN        26
  0x200f0008,         // DMA_PORT_SSI_OUT       27
  0x201d0014,         // DMA_PORT_I2C_RX        28
  0x201d0010,         // DMA_PORT_I2C_TX        29
  0,                  // EXT DREQ 0             30
  0,                  // EXT DREQ 1             31 // TODO: Clashes with DMA_PORT_ALWAYS
};
#endif

static char g_busprio[CHANNEL_CNT];
static char g_irqen[CHANNEL_CNT];

// Setup the DMA channel to a given port and priority.
// The channel will be disabled and irq is off. Irq mode is 0.
void dma_setup( int cha, int port, int busprio, int chaprio, int size, int dir)
{
  int fifo_thres;
  int mdev=MDEVADDR[port]?1:0;
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);

  g_busprio[cha]=busprio; // needed in addbuf
  g_irqen[cha]=0;

	if (cha < 5) {
		if(cha == 0)
			fifo_thres = 3;
		else
			fifo_thres = 2;
	} else
		fifo_thres = 1;
  
  channel->CHACONFIG = 
    SET_DMA_CHACONFIG_PORT(port)              |
    SET_DMA_CHACONFIG_PRIO(chaprio)           |
    SET_DMA_CHACONFIG_SIZE(size)              |
    SET_DMA_CHACONFIG_DIR(dir)                |
    SET_DMA_CHACONFIG_MDEV(mdev)|
    SET_DMA_CHACONFIG_FIFOTHRES(fifo_thres);
  
  if (mdev) {
    channel->MDEVADDR = MDEVADDR[port];
    channel->MDEVCONFIG = 
      SET_DMA_MDEVCONFIG_SIZE(0)      |
      SET_DMA_MDEVCONFIG_PROT(15)     |
      SET_DMA_MDEVCONFIG_BUSSEL(0)    |
      SET_DMA_MDEVCONFIG_PRIO(1)      |
      SET_DMA_MDEVCONFIG_INCRADDR(0)  |
      SET_DMA_MDEVCONFIG_USESIZE(0);
  }

  // Channel must be reset *after* CHACONFIG is set up
  channel->CHACTRL = SET_DMA_CHACTRL_RESET(1);
  while(channel->CHACTRL & SET_DMA_CHACTRL_RESET(1))
    ;
}

void 
dma_setup_mdev( int cha, void *mdevaddr,
		int use_xahb, int size, int dir)
{
	struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);

	g_busprio[cha] = 0; // needed in addbuf
	g_irqen[cha] = 0;

	// Configure the source-side as an MDEV which always has words
	// to read

	channel->CHACONFIG = 
		SET_DMA_CHACONFIG_PORT(DMA_PORT_ALWAYS)  |
		SET_DMA_CHACONFIG_PRIO(0)                |
		SET_DMA_CHACONFIG_SIZE(size)             |
		SET_DMA_CHACONFIG_DIR(dir)               |
		SET_DMA_CHACONFIG_MDEV(1)                |
		SET_DMA_CHACONFIG_FIFOTHRES(1);

	// Channel must be reset *after* CHACONFIG is set up to make
	// sure all states are correctly initialized

	channel->CHACTRL = SET_DMA_CHACTRL_RESET(1);
	while(channel->CHACTRL & SET_DMA_CHACTRL_RESET(1))
		;

	channel->MDEVADDR = virt_to_phys(mdevaddr);

	// Auto-increment the address, and don't use the size
	// constraint mechanism because it only allows 16-bit
	// lengths. Let it run, and the destination side will
	// take care of the length control (side-effect: the source
	// might be read up to a FIFO-length too much).

	channel->MDEVCONFIG = 
		SET_DMA_MDEVCONFIG_SIZE(0)           |
		SET_DMA_MDEVCONFIG_PROT(15)          |
		SET_DMA_MDEVCONFIG_BUSSEL(use_xahb)  |
		SET_DMA_MDEVCONFIG_PRIO(0)           |
		SET_DMA_MDEVCONFIG_INCRADDR(1)       |
		SET_DMA_MDEVCONFIG_USESIZE(0);
}

// Enable/disable the DMA channel. It is possible to 
// enable the memory and device side of the internal dma
// FIFO independently.
void dma_enable( int cha, int memory, int device)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  channel->CHACTRL = SET_DMA_CHACTRL_PAUSE(1);
  channel->CHACTRL = SET_DMA_CHACTRL_BUFEN(memory);
  channel->CHACTRL = SET_DMA_CHACTRL_DEVEN(device);
}

// Enable/disable the interrupt of a DMA channel. 
// Return previous setting.
int dma_irq_enable( int cha, int enable)
{
  // No direct correspondance in Argus III. Save and use in dma_addbuf
  int prev=g_irqen[cha];
  g_irqen[cha]=enable;
  return prev;
}

// Set IRQ mode
// 0 - buffer queue interrupt (normal case)
// 1 - memory to device transfer finish mode
void dma_irq_mode( int cha, int mode)
{
  // No correspondance in Argus III - DMA
  ASSERT(0);
}

// Add a physical memory buffer to the channel queue of buffers.
// The queue has room for up to two buffers.
void dma_add_buffer( int cha, unsigned int phys_addr, int size, int flags)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  int insert_pos=GET_DMA_CHASTATUS_DESCPOS(channel->CHASTATUS);

  // TODO: shouldn't flags be used for setting the PROT mask ?
  channel->DESCQUEUE[insert_pos].DESC0 = phys_addr;
  channel->DESCQUEUE[insert_pos].DESC1 = 
    SET_DMA_DESC1_BUFSIZE(0)     |
    SET_DMA_DESC1_PROT(15) |
    SET_DMA_DESC1_BUSSEL(flags & DMA_FLAGS_XAHB ? 1 : 0) |
    SET_DMA_DESC1_PRIO(g_busprio[cha]) |
    SET_DMA_DESC1_IRQ(g_irqen[cha]) |
    SET_DMA_DESC1_OPTION(DMA_DESC1_OPTION_REFILL) |
    SET_DMA_DESC1_INITFIFO(0) | // Prefetch fifo up to threshold before starting
    SET_DMA_DESC1_EOF(0);
  channel->DESCQUEUE[insert_pos].DESC2 = size;
  channel->DESCQUEUE[insert_pos].DESC3 = 0;
  channel->CHACTRL = SET_DMA_CHACTRL_ADDBUF(1);
}

void dma_add_buffer_stride( int cha, unsigned int phys_addr, int size, int flags, int skip, int refill)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  int insert_pos=GET_DMA_CHASTATUS_DESCPOS(channel->CHASTATUS);

  // TODO: shouldn't flags be used for setting the PROT mask ?
  channel->DESCQUEUE[insert_pos].DESC0 = phys_addr;
  channel->DESCQUEUE[insert_pos].DESC1 = 
    SET_DMA_DESC1_BUFSIZE(refill)     |
    SET_DMA_DESC1_PROT(15) |
    SET_DMA_DESC1_BUSSEL(flags & DMA_FLAGS_XAHB ? 1 : 0) |
    SET_DMA_DESC1_PRIO(g_busprio[cha]) |
    SET_DMA_DESC1_IRQ(g_irqen[cha]) |
    SET_DMA_DESC1_OPTION(DMA_DESC1_OPTION_REFILL) |
    SET_DMA_DESC1_EOF(0);
  channel->DESCQUEUE[insert_pos].DESC2 = size;
  channel->DESCQUEUE[insert_pos].DESC3 = 
	  SET_DMA_DESC3_REFILLSIZE(refill) |
	  SET_DMA_DESC3_SKIPSIZE(skip);
  channel->CHACTRL = SET_DMA_CHACTRL_ADDBUF(1);
}

// Flush buffer queue
void dma_flush_buffer( int cha)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  // Reset channel. Not possible to do flush only.
  channel->CHACTRL = SET_DMA_CHACTRL_RESET(1);
  while(channel->CHACTRL & SET_DMA_CHACTRL_RESET(1))
    ;
}

// Drain dma channel fifo.
// return 1 if successful drain
// return 0 if out of buffers in the buffer queue
int dma_drain( int cha)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  channel->CHACTRL = SET_DMA_CHACTRL_DRAIN(1);
  while((channel->CHACTRL & SET_DMA_CHACTRL_DRAIN(1)) && 
        GET_DMA_CHASTATUS_DESCCNT(channel->CHASTATUS)>0)
    ;
  return !(channel->CHACTRL & SET_DMA_CHACTRL_DRAIN(1));
}

// Get buffer queue status.
// *phys_addr = current address pointer in buffer. The
// address is only valid if the buffer count is 1 or 2.
// Return number of buffers in the queue (0,1 or 2)
int dma_buffer_status( int cha, unsigned int *phys_addr)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  int status=channel->CHASTATUS;
  int pos = GET_DMA_CHASTATUS_DESCPOS(status);
  int cnt = GET_DMA_CHASTATUS_DESCCNT(status);
  int cur = (2+pos-cnt)&1;
  if (phys_addr)
    *phys_addr=channel->DESCQUEUE[cur].DESC0;
  return cnt;
}

// Return number of bytes currently in the channel fifo
int dma_fifo_status( int cha)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  int status=channel->CHASTATUS;
  return GET_DMA_CHASTATUS_FIFOFILL(status);
}  

// Flush fifo
void dma_flush_fifo( int cha)
{
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  // Reset channel. Not possible to do flush only.
  channel->CHACTRL = SET_DMA_CHACTRL_RESET(1);
  while(channel->CHACTRL & SET_DMA_CHACTRL_RESET(1))
    ;
}

// Abort the dma transfer on the given channel.
// The fifo and the buffer queue is not flushed.
// This must be done using the dma_flush_fifo and
// the dma_flush_buffer commands.
// The channel irq is disabled.
void dma_abort( int cha)
{ 
  struct tDMA_CHANNEL *channel= DMA_CHANNEL(cha);
  // Reset channel. //TODO: maybe we would like to keep some of the state. Use PAUSE instead?
  channel->CHACTRL = SET_DMA_CHACTRL_RESET(1);
  while(channel->CHACTRL & SET_DMA_CHACTRL_RESET(1))
    ;
}

// Returns the lowest dma channel generating interrupt.
// Returns -1 if no interrupt generated.
int dma_get_irq(void)
{
#ifdef CONFIG_ARGUS_3
	int irq = SOC_DMA.IRQ;
	if(irq)
		return GET_DMA_IRQ_CHANNEL(irq);
	else
		return -1;
#else
  int dma, lowest= -1;
  for (dma= 0; dma<2; dma++)
  {
    int irq = SOC_DMA[dma].IRQ;
    if(irq)
    {
      int cha= 2*GET_DMA_IRQ_CHANNEL(irq)+dma;
      if (lowest==-1 || cha<lowest)
        lowest= cha;
    }
  }
  return lowest;
#endif
}

EXPORT_SYMBOL(dma_add_buffer);
EXPORT_SYMBOL(dma_add_buffer_stride);
EXPORT_SYMBOL(dma_buffer_status);
EXPORT_SYMBOL(dma_setup);
EXPORT_SYMBOL(dma_drain);
EXPORT_SYMBOL(dma_abort);

