/*
 *  linux/arch/arm/mach-argus/mm.c
 *
 *  Copyright (C) 2003,2004 Anoto Group AB and Axis Communications AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/mm.h>
#include <linux/init.h>

#include <asm/mach/map.h>

#include <asm/arch/argus_addr.h>

// Table of initial global kernel mappings. The mm stuff will handle the
// difference between page mappings and section mappings and use the latter
// where applicable, so it is more efficient to map whole 1 MB areas.
//
// The flash is mapped both cached and uncached.

static struct map_desc argus_io_desc[] __initdata = {
	{       /* CS0, Flash 32 MB */
		.virtual = EXT0_NOCACHE_V,
		.pfn     = __phys_to_pfn(EXT0_BASE_P),
		.length  = 0x02000000,
		.type    = MT_DEVICE
	}, {    /* CS0, Flash 32 MB */
		.virtual = EXT0_CACHE_V,
		.pfn     = __phys_to_pfn(EXT0_BASE_P),
		.length  = 0x02000000,
		.type    = MT_MEMORY
	}, {    /* APB 2 MB */
		.virtual = APB_NOCACHE,
		.pfn     = __phys_to_pfn(APB_BASE_P),
		.length  = 0x00200000,
		.type    = MT_DEVICE
	}, {    /* OCRAM 1 MB */
		.virtual = OCRAM_BASE_V,
		.pfn     = __phys_to_pfn(OCRAM_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	}, {    /* L2 CTRL 1MB */
		.virtual = L2CACHE_CTRL_NOCACHE,
		.pfn     = __phys_to_pfn(L2CACHE_CTRL_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
#ifdef CONFIG_ARGUS_3
	// This is already mapped as OCRAM_BASE on Argus-4
	{       /* L2 1MB */
		.virtual = L2CACHE_STAT_BASE_V,
		.pfn     = __phys_to_pfn(L2CACHE_STAT_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
#endif
	{       /* DMA in Argus3 1 MB */
		.virtual = DMA_NOCACHE,
		.pfn     = __phys_to_pfn(DMA_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	}, {    /* EXT2 nocache 1MB */
		.virtual = EXT2_NOCACHE_V,
		.pfn     = __phys_to_pfn(EXT2_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	}, {    /* EXT4 (SDRAM) nocache 128MB */
		.virtual = EXT4_NOCACHE_V,
		.pfn     = __phys_to_pfn(EXT4_BASE_P),
		.length  = 0x08000000,
		.type    = MT_DEVICE
	},
#ifdef CONFIG_ARGUS_4
	{       /* VSYS nocache 1MB */
		.virtual = VSYS_NOCACHE,
		.pfn     = __phys_to_pfn(VSYS_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
	{       /* Ethernet nocache 1MB */
		.virtual = ETH_NOCACHE,
		.pfn     = __phys_to_pfn(ETH_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
	{       /* AES nocache 1MB */
		.virtual = AES_NOCACHE,
		.pfn     = __phys_to_pfn(AES_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
#endif
#ifdef CONFIG_ARGUS_3	
	/* The following weird mappings are for the 206W CF-card interface */
	
	{       /* EXT5 nocache */
		.virtual = EXT5_NOCACHE1_V,
		.pfn     = __phys_to_pfn(EXT5_BASE_P + 0x800000),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	}, {    /* EXT5 nocache */
		.virtual = EXT5_NOCACHE2_V,
		.pfn     = __phys_to_pfn(EXT5_BASE_P + 0x500000),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	}, {    /* EXT5 nocache */
		.virtual = EXT5_NOCACHE3_V,
		.pfn     = __phys_to_pfn(EXT5_BASE_P + 0x400000),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
	
	/* The following mapping is for NAND-flash, attached to CS5 with A18 to CLE and A19 to ALE */
	
	{       /* EXT5 nocache */
		.virtual = NAND_NOCACHE_DATA_V,
		.pfn     = __phys_to_pfn(EXT5_BASE_P),
		.length  = 0x00100000,
		.type    = MT_DEVICE
	},
#endif /* CONFIG_ARGUS_3 */
};

void __init 
argus_map_io(void)
{
        iotable_init(argus_io_desc, ARRAY_SIZE(argus_io_desc));
}
