/*
 * Taken from arch/cris/arch-v32/drivers/nandflash.c
 * and modified to use for boot loader.
 *
 *  Copyright (c) 2004
 *
 *  Derived from drivers/mtd/nand/spia.c
 * 	  Copyright (C) 2000 Steven J. Hill (sjhill@realitydiluted.com)
 * 
 * $Id: crisv32_nand.c,v 1.2 2006/11/21 14:40:02 ricardw Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include "mtd.h"
#include "nand.h"

#if 0
#include <linux/mtd/partitions.h>
#endif

#if 0
#include <asm/arch/memmap.h>
#endif

#include <asm/arch/hwregs/reg_map.h>
#include <asm/arch/hwregs/reg_rdwr.h>
#include <asm/arch/hwregs/gio_defs.h>
#include <asm/arch/hwregs/bif_core_defs.h>

#include "lib.h"

/* Hardware */

#define CE_BIT 4
#define CLE_BIT 5
#define ALE_BIT 6
#define BY_BIT 7

#define NAND_RD_ADDR 0x90000000 /* read address */
#define NAND_WR_ADDR 0x94000000 /* write address */

static struct mtd_info *crisv32_mtd = NULL;

/* 
 *	hardware specific access to control-lines
 */
static void crisv32_hwcontrol(struct mtd_info *mtd, int cmd)
{
	unsigned long flags;
	reg_gio_rw_pa_dout dout = REG_RD(gio, regi_gio, rw_pa_dout);

	switch(cmd){
		case NAND_CTL_SETCLE: 
		     dout.data |= (1<<CLE_BIT);
		     break;
		case NAND_CTL_CLRCLE: 
		     dout.data &= ~(1<<CLE_BIT);
		     break;
		case NAND_CTL_SETALE:
		     dout.data |= (1<<ALE_BIT);
		     break;
		case NAND_CTL_CLRALE: 
		     dout.data &= ~(1<<ALE_BIT);
		     break;
		case NAND_CTL_SETNCE:
		     dout.data &= ~(1<<CE_BIT);
		     break;
		case NAND_CTL_CLRNCE:
		     dout.data |= (1<<CE_BIT);
		     break;
	}
	REG_WR(gio, regi_gio, rw_pa_dout, dout);
#if 0 
	/* read from gpio reg to flush pipeline.
	 * doesn't seem to be necessary.
	 */
	(void) REG_RD(gio, regi_gio, rw_pa_dout); /* gpio sync */
#endif
}

/*
 *	read device ready pin
 */
int crisv32_device_ready(struct mtd_info *mtd)
{
	reg_gio_r_pa_din din = REG_RD(gio, regi_gio, r_pa_din);
	return ((din.data & (1 << BY_BIT)) >> BY_BIT);
}

/*
 * Main initialization routine
 */
struct mtd_info* crisv32_nand_flash_probe (void)
{
	reg_bif_core_rw_grp3_cfg bif_cfg = REG_RD(bif_core, regi_bif_core, rw_grp3_cfg);
	reg_gio_rw_pa_oe pa_oe = REG_RD(gio, regi_gio, rw_pa_oe);
	struct nand_chip *this;
	int err = 0;

	/* Allocate memory for MTD device structure and private data */
	crisv32_mtd = malloc (sizeof(struct mtd_info) + sizeof (struct nand_chip));
	if (!crisv32_mtd) {
		puts ("Unable to allocate CRISv32 NAND MTD device structure.\r\n");
		err = -ENOMEM;
		return NULL;
	}

	/* Get pointer to private data */
	this = (struct nand_chip *) (&crisv32_mtd[1]);

	pa_oe.oe |= 1 << CE_BIT;
	pa_oe.oe |= 1 << ALE_BIT;
	pa_oe.oe |= 1 << CLE_BIT;
	pa_oe.oe &= ~ (1 << BY_BIT);
	REG_WR(gio, regi_gio, rw_pa_oe, pa_oe);

	bif_cfg.gated_csp0 = regk_bif_core_rd;
	bif_cfg.gated_csp1 = regk_bif_core_wr;
	REG_WR(bif_core, regi_bif_core, rw_grp3_cfg, bif_cfg);

	/* Initialize structures */
	memset((char *) crisv32_mtd, 0, sizeof(struct mtd_info));
	memset((char *) this, 0, sizeof(struct nand_chip));

	/* Link the private data with the MTD structure */
	crisv32_mtd->priv = this;

	/* Set address of NAND IO lines */
	this->IO_ADDR_R = (void *) NAND_RD_ADDR;
	this->IO_ADDR_W = (void *) NAND_WR_ADDR;
	this->hwcontrol = crisv32_hwcontrol;
	this->dev_ready = crisv32_device_ready;
	/* 20 us command delay time */
	this->chip_delay = 20;		
	this->eccmode = NAND_ECC_SOFT;

#if 0 /* don't use BBT in boot loader */
	/* Enable the following for a flash based bad block table */
	this->options = NAND_USE_FLASH_BBT;
#endif
	/* don't scan for BBT */
	this->options = NAND_SKIP_BBTSCAN;

	/* Scan to find existance of the device */
	if (nand_scan (crisv32_mtd, 1)) {
		err = -ENXIO;
		puts ("nand_scan failed\r\n");
		free (crisv32_mtd);
		return NULL;
	}
	
	return crisv32_mtd;
}
