/*
 * lib.c
 * Small practical functions for boot loader
 * malloc/free
 * memset/memcpy
 * putc/puts/putnybble/putd
 * writeb/writew/readb/readw
 * error/error2/BUG
 * serial_init
 *
 * $Id: lib.h,v 1.3 2006/11/03 10:35:52 pkj Exp $
 * 
 */

#ifndef _LIB_H
#define _LIB_H

#include <linux/types.h>

/* nice stuff we need without having any library around */

void* memset(void* s, int c, size_t n);
void* memcpy(void* __dest, __const void* __src,
	     size_t __n);

#define memzero(s, n)     memset ((s), 0, (n))

#undef BUG
#define BUG() error2("BUG in " __FILE__, __LINE__, __FUNCTION__)

void *malloc(int size);
void free(void *where);
void error(const char *m);
void error2(const char *m, int l, const char *f);
void serial_init(void);

#ifndef CONFIG_ETRAX_DEBUG_PORT_NULL 
void putc(const char);
void puts(const char *);
void putnybble(int n);
void putx(int x);
void putnl();
#else
#define putc(ch) 
#define puts(str)
#define putnybble(nyb)
#define putx(x)
#define putnl()
#endif /* CONFIG_ETRAX_DEBUG_PORT_NULL */

unsigned char readb(const volatile void *addr);
unsigned short readw(const volatile void *addr);
void writeb(unsigned char b, volatile void *addr);
void writew(unsigned short b, volatile void *addr);

#endif /* _LIB_H */
