/*
 * Definitions for ETRAX FS ethernet driver.
 *
 * Copyright (C) 2003, 2004, 2005 Axis Communications.
 */

#ifndef _ETRAX_ETHERNET_H_
#define _ETRAX_ETHERNET_H_

#include <asm/arch/hwregs/dma.h>


#define MAX_MEDIA_DATA_SIZE 1522	/* Max packet size. */

#define NBR_RX_DESC 64			/* Number of RX descriptors. */
#define NBR_TX_DESC 16			/* Number of TX descriptors. */
#ifdef CONFIG_CRIS_MACH_ARTPEC3
#define NBR_INTMEM_RX_DESC 5		/* Number of RX descriptors in int. mem.
					 * when running in gigabit mode.
					 * Should be less then NBR_RX_DESC 
					 */
#define NBR_INTMEM_TX_BUF 4		/* Number of TX buffers in int. mem
					 * when running in gigabit mode.
					 * Should be less than NBR_TX_DESC
					 */
#endif

/* Large packets are sent directly to upper layers while small packets
 * are copied (to reduce memory waste).  The following constant
 * decides the breakpoint.
 */
#define RX_COPYBREAK        (256)

#define ETHER_HEAD_LEN      (14)

/* 
** MDIO constants.
*/
#define MDIO_START                          0x1
#define MDIO_READ                           0x2
#define MDIO_WRITE                          0x1
#define MDIO_PREAMBLE              0xfffffffful

/* Broadcom specific */
#define MDIO_AUX_CTRL_STATUS_REG           0x18
#define MDIO_BC_FULL_DUPLEX_IND             0x1
#define MDIO_BC_SPEED                       0x2

/* TDK specific */
#define MDIO_TDK_DIAGNOSTIC_REG              18
#define MDIO_TDK_DIAGNOSTIC_RATE          0x400
#define MDIO_TDK_DIAGNOSTIC_DPLX          0x800

/*Intel LXT972A specific*/
#define MDIO_INT_STATUS_REG_2		   0x0011
#define MDIO_INT_FULL_DUPLEX_IND ( 0x0001 << 9  )
#define MDIO_INT_SPEED		 ( 0x0001 << 14 )

/*National Semiconductor DP83865 specific*/
#define MDIO_NAT_LINK_AN_REG              0x11
#define MDIO_NAT_1000            (0x0001 << 4)
#define MDIO_NAT_100             (0x0001 << 3)
#define MDIO_NAT_FULL_DUPLEX_IND (0x0001 << 1)

/* Network flash constants */
#define NET_FLASH_TIME                  (HZ/50) /* 20 ms */
#define NET_FLASH_PAUSE                 (HZ/100) /* 10 ms */
#define NET_LINK_UP_CHECK_INTERVAL	(2*HZ)	/* 2 seconds. */
#define NET_DUPLEX_CHECK_INTERVAL	(2*HZ)	/* 2 seconds. */

/* Duplex settings. */
enum duplex {
	half,
	full,
	autoneg
};

/* Some transceivers requires special handling. */
struct transceiver_ops {
	unsigned int oui;
	void (*check_speed) (struct net_device * dev);
	void (*check_duplex) (struct net_device * dev);
};

typedef struct crisv32_eth_descr {
	dma_descr_data descr __attribute__ ((__aligned__(32)));
	struct sk_buff *skb;
	unsigned char *linearized_packet;
} crisv32_eth_descr;



#ifdef CONFIG_CRIS_MACH_ARTPEC3
struct tx_buffer_list {
  struct tx_buffer_list *next;
  unsigned char *buf;
  char free;
};
#endif

/* LED stuff */
#define LED_GRP_0 0
#define LED_GRP_1 1
#define LED_GRP_NONE 2

#define LED_ACTIVITY   0
#define LED_NOACTIVITY 1
#define LED_LINK       2
#define LED_NOLINK     3

struct crisv32_eth_leds {
	unsigned int ledgrp;
	int led_active;
	unsigned long led_next_time;
	struct timer_list clear_led_timer;
	spinlock_t led_lock; /* Protect LED state */
	int ifisup[2];
};

#define NET_LED_SET(x,y)				\
	do {						\
		if (x == 0) LED_NETWORK_GRP0_SET(y);	\
		if (x == 1) LED_NETWORK_GRP1_SET(y);	\
	} while (0)

/* Information that need to be kept for each device. */
struct crisv32_ethernet_local {
	dma_descr_context ctxt_in __attribute__ ((__aligned__(32)));
	dma_descr_context ctxt_out __attribute__ ((__aligned__(32)));

	crisv32_eth_descr *active_rx_desc;
	crisv32_eth_descr *prev_rx_desc;
	crisv32_eth_descr *last_rx_desc;

	crisv32_eth_descr *active_tx_desc;
	crisv32_eth_descr *prev_tx_desc;
	crisv32_eth_descr *catch_tx_desc;

	crisv32_eth_descr dma_rx_descr_list[NBR_RX_DESC];
	crisv32_eth_descr dma_tx_descr_list[NBR_TX_DESC];
#ifdef CONFIG_CRIS_MACH_ARTPEC3
	struct tx_buffer_list tx_intmem_buf_list[NBR_INTMEM_TX_BUF];
	struct tx_buffer_list *intmem_tx_buf_active;
	struct tx_buffer_list *intmem_tx_buf_catch;
	char gigabit_mode;
#endif
	char new_rx_package;

	/* DMA and ethernet registers for the device. */
	int eth_inst;
	int dma_in_inst;
	int dma_out_inst;

	/* Network speed indication. */
	struct timer_list speed_timer;
	int current_speed;		/* Speed read from tranceiver */
	int current_speed_selection;	/* Speed selected by user */
	int sender_started;
	int txpackets;

	struct crisv32_eth_leds *leds;

	/* Duplex. */
	struct timer_list duplex_timer;
	int full_duplex;
	enum duplex current_duplex;

	struct net_device_stats stats;

	/* Transciever address. */
	unsigned int mdio_phy_addr;

	struct transceiver_ops *transceiver;

	/* 
	 * TX control lock. This protects the transmit buffer ring state along
	 * with the "tx full" state of the driver.  This means all netif_queue
	 * flow control actions are protected by this lock as well.
	 */
	spinlock_t lock;
	spinlock_t transceiver_lock; /* Protect transceiver state. */
};

/* Function prototypes. */
static int crisv32_ethernet_init(void);
static int crisv32_ethernet_device_init(struct net_device* dev);
static int crisv32_eth_open(struct net_device *dev);
static int crisv32_eth_close(struct net_device *dev);
static int crisv32_eth_set_mac_address(struct net_device *dev, void *vpntr);
static irqreturn_t crisv32rx_eth_interrupt(int irq, void *dev_id);
static irqreturn_t crisv32tx_eth_interrupt(int irq, void *dev_id);
static irqreturn_t crisv32nw_eth_interrupt(int irq, void *dev_id);
static void crisv32_eth_receive_packet(struct net_device *dev);
static int crisv32_eth_send_packet(struct sk_buff *skb, struct net_device *dev);
static void crisv32_eth_hw_send_packet(unsigned char *buf, int length,
				       void *priv);
static void crisv32_eth_tx_timeout(struct net_device *dev);
static void crisv32_eth_set_multicast_list(struct net_device *dev);
static int crisv32_eth_ioctl(struct net_device *dev, struct ifreq *ifr,
			     int cmd);
static int crisv32_eth_set_config(struct net_device* dev, struct ifmap* map);
#ifdef CONFIG_CRIS_MACH_ARTPEC3
static void crisv32_eth_switch_intmem_usage(struct net_device *dev);
#endif
static void crisv32_eth_negotiate(struct net_device *dev);
static void crisv32_eth_check_speed(unsigned long idev);
static void crisv32_eth_set_speed(struct net_device *dev, unsigned long speed);
static void crisv32_eth_check_duplex(unsigned long idev);
static void crisv32_eth_set_duplex(struct net_device *dev, enum duplex);
static int crisv32_eth_probe_transceiver(struct net_device *dev);

static struct ethtool_ops crisv32_ethtool_ops;

static void generic_check_speed(struct net_device *dev);
static void generic_check_duplex(struct net_device *dev);
static void broadcom_check_speed(struct net_device *dev);
static void broadcom_check_duplex(struct net_device *dev);
static void tdk_check_speed(struct net_device *dev);
static void tdk_check_duplex(struct net_device *dev);
static void intel_check_speed(struct net_device* dev);
static void intel_check_duplex(struct net_device *dev);
static void national_check_speed(struct net_device* dev);
static void national_check_duplex(struct net_device *dev);

#ifdef CONFIG_NET_POLL_CONTROLLER
static void crisv32_netpoll(struct net_device* dev);
#endif

static void crisv32_clear_network_leds(unsigned long dummy);
static void crisv32_set_network_leds(int active, struct net_device* dev);

static void crisv32_eth_reset_tranceiver(struct net_device *dev);
static unsigned short crisv32_eth_get_mdio_reg(struct net_device *dev,
					       unsigned char reg_num);
static void crisv32_eth_set_mdio_reg(struct net_device *dev,
                                     unsigned char reg_num,
                                     int val);
static void crisv32_eth_send_mdio_cmd(struct net_device *dev,
				      unsigned short cmd, int write_cmd);
static void crisv32_eth_send_mdio_bit(struct net_device *dev,
				      unsigned char bit);
static unsigned char crisv32_eth_receive_mdio_bit(struct net_device *dev);

static struct net_device_stats *crisv32_get_stats(struct net_device *dev);
static void crisv32_start_dma_out(struct crisv32_ethernet_local* np);


#endif /* _ETRAX_ETHERNET_H_ */
