/*
 *  Copyright (C) 2003,2004,2005,2006,2007 Anoto Group AB and Axis Communications AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef ARGUS4_H
#define ARGUS4_H

// Correct pagetable is always generated in the include-path!
// Defines the virtual address space in terms of xxx_NOCACHE, xxx_CACHE etc constants
#include <asm/arch/argus_addr.h>

#define REG32 volatile unsigned int

//--------------------------------------------------------------------------------
// Use definitions below to access SOC hardware
//--------------------------------------------------------------------------------

#define SOC_RESET     (*(tRESET*)(APB_NOCACHE + 0x00000)   )  // Reset controller,
#define SOC_CLK       (*(tCLK*)(APB_NOCACHE + 0x10000)     )  // Clock controller,
#define SOC_IRQ       (*(tIRQ*)(APB_NOCACHE + 0x20000)     )  // IRQ controller for ARM
#define SOC_TIMER     (*(tTIMER*)(APB_NOCACHE + 0x30000)   )  // Timer,
#define SOC_GPIO      (*(tGPIO*)(APB_NOCACHE + 0x40000)    )  // General purpose I/O,
#define SOC_PWM       ( (tPWM*)(APB_NOCACHE + 0x50000)     )  // Pulse wave modulators (2),
#define SOC_AVP       (*(tAVP*)(APB_NOCACHE + 0x70000)     )  // Video Processor block,
#define SOC_PINCTRL   (*(tPINCTRL*)(APB_NOCACHE + 0x80000) )  // Pin controller
#define SOC_DDR       (*(tDDR*)(APB_NOCACHE + 0x90000)  )     // DDR controller
#define SOC_EXTMEM    (*(tEXTMEM*)(APB_NOCACHE + 0xa0000)  )  // External memory interface,
#define SOC_UART      ( (tUART*)(APB_NOCACHE + 0xb0000)    )  // UART's (2)
#define SOC_USB       (*(tUSB*)(APB_NOCACHE + 0xd0000)     )  // USB slave
#define SOC_SSI       (*(tSSI*)(APB_NOCACHE + 0xf0000)     )  // Synchronous Serial Interface
#define SOC_SDHC      ( (tSDHC*)(APB_NOCACHE + 0x100000)   )  // SD-card Host Controllers (2),
#define SOC_CLKGEN    ( (tCLKGEN*)(APB_NOCACHE + 0x120000) )  // Clock generators (4),
#define SOC_VIDOUT    (*(tVIDOUT*)(APB_NOCACHE + 0x160000) )  // Video output
#define SOC_EXTDMA    (*(tEXTDMA*)(APB_NOCACHE + 0x170000) )  // Extern DMA
#define SOC_XARB      (*(tXARB*)(APB_NOCACHE + 0x180000)   )  // XARB
#define SOC_SMIA      (*(tSMIA*)(APB_NOCACHE + 0x190000)   )  // SMIA interface
#define SOC_HOSTIF    (*(tHOSTIF*)(APB_NOCACHE + 0x1a0000) )  // Host interface
#define SOC_RASC      (*(tRASC*)(APB_NOCACHE + 0x1b0000)   )  // RASC control registers
#define SOC_JPEG      (*(tJPEG*)(APB_NOCACHE + 0x1c0000)   )  // JPEG encoder
#define SOC_I2C       (*(tI2C*)(APB_NOCACHE + 0x1d0000)    )  // I2C interface
#define SOC_MBDMA     (*(tMBDMA*)(APB_NOCACHE + 0x1e0000)  )  // Macroblock DMA
#define SOC_FPGA      (*(tFPGA*)(APB_NOCACHE + 0x1f0000)   )  // FPGA specific registers
#define SOC_IRQRASC   (*(tIRQ*)(APB_NOCACHE + 0x200000)    )  // IRQ controllor for RASC
#define SOC_L2CACHE   (*(tL2CACHE*)(L2CACHE_CTRL_NOCACHE)  )  // L2-Cache Control Interface
#define SOC_AVE       (*(tAVE*)(VSYS_NOCACHE + 0x10000) 	 )  // AVE
#define SOC_AVE_PHYS  (*(tAVE*)(VSYS_BASE_P + 0x10000)    	 )  // AVE, physical adress
#define SOC_AES       (*(tAES*)(AES_NOCACHE)               )  // AES
#define SOC_ETHERNET  (*(tETHERNET*)(ETH_NOCACHE)          )  // Ethernet
#define SOC_DMA       ( (tDMA*)(DMA_NOCACHE)               )  // DMA controllers (2)

//--------------------------------------------------------------------------------
// Type definitions of hardware interface structures
//--------------------------------------------------------------------------------

//-----------------------------
// Reset controller
//-----------------------------

typedef struct _tRESET
{
  REG32 SWRESET;
  REG32 WATCHDOG;
  REG32 REASON;
                #define RESET_REASON_HW         0
                #define RESET_REASON_SW         1
                #define RESET_REASON_WD         2
} tRESET;

//-----------------------------
// Clock controller
//-----------------------------

typedef struct _tCLK
{
  REG32 CTRL;                           // Clock Controller Control
    #define GET_CLK_HISPD(x) ((x>>0)&0x1)             // set for high speed (pll) mode
    #define CHG_CLK_HISPD(x,upd) (x&~0x1 | (upd<<0))
    #define SET_CLK_HISPD(upd) (upd<<0)
    #define GET_CLK_HDIV(x) ((x>>1)&0x3)              // HCLK divider (PHYSCACHE,IMAGE,JPEG,MATCH,MONTY...)
    #define CHG_CLK_HDIV(x,upd) (x&~0x6 | (upd<<1))
    #define SET_CLK_HDIV(upd) (upd<<1)

    #define GET_CLK_OSC(x) ((x>>3)&0x3)             // set for 13 MHz oscilator, clear for 12 MHz.
    #define CHG_CLK_OSC(x,upd) (x&~0x18 | (upd<<3))
    #define SET_CLK_OSC(upd) (upd<<3)

    #define GET_CLK_OSCTIME(x) ((x>>5)&0x1f)          // settling time for the oscilator when leaving sleep mode (osc disabled),
                                                      // 0-17, 17 => ~10 ms, 16 => ~5 ms ...
    #define CHG_CLK_OSCTIME(x,upd) (x&~0x03e0 | (upd<<5))
    #define SET_CLK_OSCTIME(upd) (upd<<5)

    #define GET_CLK_EXT_OSC_CTRL(x) ((x>>10)&0x1)     // Mux OSCEN to BT_SYS_CLK_REQ pin
    #define SET_CLK_EXT_OSC_CTRL(upd) (upd<<10)
    #define GET_CLK_HISPD_STATUS(x) ((x>>11)&0x1) // read only bit, used for checking the status of a HISPD change

  REG32 PLL;                            // PLL Control
    #define GET_PLL_E(x) ((x>>0)&0x1)
    #define CHG_PLL_E(x,upd) (x&~0x1 | (upd<<0))
    #define SET_PLL_E(upd) (upd<<0)
    #define GET_PLL_FBDIV(x) ((x>>1)&0x7f)
    #define CHG_PLL_FBDIV(x,upd) (x&~0xfe | (upd<<1))
    #define SET_PLL_FBDIV(upd) (upd<<1)
    #define GET_PLL_PDIV(x) ((x>>8)&0xf)
    #define CHG_PLL_PDIV(x,upd) (x&~0xf00 | (upd<<8))
    #define SET_PLL_PDIV(upd) (upd<<8)
    #define GET_PLL_LBWS(x) ((x>>12)&0x3)
    #define CHG_PLL_LBWS(x,upd) (x&~0x3000 | (upd<<12))
    #define SET_PLL_LBWS(upd) (upd<<12)
    #define GET_PLL_LFRESN(x) ((x>>14)&0x1)
    #define CHG_PLL_LFRESN(x,upd) (x&~0x4000 | (upd<<14))
    #define SET_PLL_LFRESN(upd) (upd<<14)
    #define GET_PLL_HALFSPEED(x) ((x>>15)&0x1)
    #define CHG_PLL_HALFSPEED(x,upd) (x&~0x8000 | (upd<<15))
    #define SET_PLL_HALFSPEED(upd) (upd<<15)
    #define GET_PLL_CLK450_EN(x) ((x>>16)&0x1)
    #define CHG_PLL_CLK450_EN(x,upd) (x&~0x10000 | (upd<<16))
    #define SET_PLL_CLK450_EN(upd) (upd<<16)
    #define GET_PLL_LFBREN(x) ((x>>17)&0x1)
    #define CHG_PLL_LFBREN(x,upd) (x&~0x20000 | (upd<<17))
    #define SET_PLL_LFBREN(upd) (upd<<17)

  REG32 ENABLE;                          // Clock Gating read and enable (equal to CCG)
  REG32 DISABLE;                         // Clock Gating Disable
    #define CLK_CPU      (1<<0)          // CPU Clock
    #define CLK_AHB      (1<<1)          // AHB Clock
    #define CLK_APB      (1<<2)          // APB Clock
    #define CLK_DMA      (1<<3)          // DMA clock
    #define CLK_VSYS     (1<<4)          // Video subsystem clock
    #define CLK_ETH      (1<<5)          // Ethernet Clock
    #define CLK_SDHC     (1<<6)          // SD host controller clock
    #define CLK_APB_CONT (1<<7)          // PCLK_CONT
    #define CLK_OSC      (1<<8)          // Oscillator Enable
    #define CLK_UART0    (1<<9)          // UART 0 Clock
    #define CLK_UART1    (1<<10)          // UART 1 Clock
    #define CLK_CLKGEN0  (1<<11)         // Clock Generator 0 Clock
    #define CLK_CLKGEN1  (1<<12)         // Clock Generator 1 Clock
    #define CLK_CLKGEN2  (1<<13)         // Clock Generator 2 Clock
    #define CLK_CLKGEN3  (1<<14)         // Clock Generator 3 Clock
    #define CLK_SSI      (1<<15)         // Serial Synchronous Interface (SSI) Clock
    #define CLK_EXTMEM   (1<<16)         // External Memory Clock
    #define CLK_USB      (1<<17)         // Normal USB clock
    #define CLK_USB_TEST (1<<18)         // USB prod test mode. NOTE: Both CLK_USB and CLK_USB_TEST should not be enabled!!
    #define CLK_AES      (1<<19)         // AES clock
    #define CLK_I2C      (1<<20)         // I2C clock
    #define CLK_HOSTIF   (1<<21)         // HOSTIF clock
    
  REG32 CLKDIVPER;
  REG32 CLKREQ;
    #define SET_CLKREQ_ASYNC_DMA(upd) (upd<<0)
    #define GET_CLKREQ_ASYNC_DMA(x) ((x>>0)&0x1)
    #define SET_CLKREQ_ASYNC_GPIO(upd) (upd<<1)
    #define GET_CLKREQ_ASYNC_GPIO(x) ((x>>1)&0x1)
    #define SET_CLKREQ_ASYNC_USB(upd) (upd<<2)
    #define GET_CLKREQ_ASYNC_USB(x) ((x>>2)&0x1)
//    #define SET_CLKREQ_ASYNC_RTC(upd) (upd<<3)
//    #define GET_CLKREQ_ASYNC_RTC(x) ((x>>3)&0x1)
    #define SET_CLKREQ_CCLK(upd) (upd<<4)
    #define GET_CLKREQ_CCLK(x) ((x>>4)&0x1)
//    #define SET_CLKREQ_PCLK_BT(upd) (upd<<5)
//    #define GET_CLKREQ_PCLK_BT(x) ((x>>5)&0x1)
    #define SET_CLKREQ_PCLK_USB(upd) (upd<<6)
    #define GET_CLKREQ_PCLK_USB(x) ((x>>6)&0x1)
//    #define SET_CLKREQ_ASYNC_BT(upd) (upd<<7)
//    #define GET_CLKREQ_ASYNC_BT(x) ((x>>7)&0x1)
    #define SET_CLKREQ_TLBCLK(upd) (upd<<8)
    #define GET_CLKREQ_TLBCLK(x) ((x>>8)&0x1)
    #define SET_CLKREQ_EXTCLK(upd) (upd<<9)
    #define GET_CLKREQ_EXTCLK(x) ((x>>9)&0x1)
    #define SET_CLKREQ_PCLK_RESET(upd) (upd<<10)
    #define GET_CLKREQ_PCLK_RESET(x) ((x>>10)&0x1)
    #define SET_CLKREQ_PCLK_BRIDGE(upd) (upd<<11)
    #define GET_CLKREQ_PCLK_BRIDGE(x) ((x>>11)&0x1)
    #define SET_CLKREQ_PCLK_PWM0(upd) (upd<<12)
    #define GET_CLKREQ_PCLK_PWM0(x) ((x>>12)&0x1)
    #define SET_CLKREQ_PCLK_PWM1(upd) (upd<<13)
    #define GET_CLKREQ_PCLK_PWM1(x) ((x>>13)&0x1)
    #define SET_CLKREQ_PCLK_TIMER(upd) (upd<<14)
    #define GET_CLKREQ_PCLK_TIMER(x) ((x>>14)&0x1)
    #define SET_CLKREQ_PCLK_GPIO(upd) (upd<<15)
    #define GET_CLKREQ_PCLK_GPIO(x) ((x>>15)&0x1)
    #define SET_CLKREQ_PCLK_AVP(upd) (upd<<16)
    #define GET_CLKREQ_PCLK_AVP(x) ((x>>16)&0x1)
    #define SET_CLKREQ_L2CACHE(upd) (upd<<17)
    #define GET_CLKREQ_L2CACHE(x) ((x>>17)&0x1)
    #define SET_CLKREQ_AHB(upd) (upd<<18)
    #define GET_CLKREQ_AHB(x) ((x>>18)&0x1)
    #define SET_CLKREQ_DMA(upd) (upd<<19)
    #define GET_CLKREQ_DMA(x) ((x>>19)&0x1)
    #define SET_CLKREQ_SDRAM(upd) (upd<<20)
    #define GET_CLKREQ_SDRAM(x) ((x>>20)&0x1)
    #define SET_CLKREQ_APB(upd) (upd<<21)
    #define GET_CLKREQ_APB(x) ((x>>21)&0x1)
    #define SET_CLKREQ_OCRAM(upd) (upd<<22)
    #define GET_CLKREQ_OCRAM(x) ((x>>22)&0x1)
    #define SET_CLKREQ_AHB_LOWPOWER(upd) (upd<<23)
    #define GET_CLKREQ_AHB_LOWPOWER(x) ((x>>23)&0x1)
    #define SET_CLKREQ_ASYNC_EXTMEM(upd) (upd<<24)
    #define GET_CLKREQ_ASYNC_EXTMEM(x) ((x>>24)&0x1)

} tCLK;

//-----------------------------
// Interrupt controller
//-----------------------------

typedef struct _tIRQ
{
  REG32 IRQ_SRCS;     //IQIP;            // Irq Pending (after mask and level)
  REG32 MASK;         //IQMR;            // Mask Register
    #define IRQ_UART0_MASK    0x00000001
    #define IRQ_UART1_MASK    0x00000002
    #define IRQ_TIMER0_MASK   0x00000004
    #define IRQ_TIMER1_MASK   0x00000008
    #define IRQ_TIMER2_MASK   0x00000010
    #define IRQ_TIMER3_MASK   0x00000020
    #define IRQ_GPIO_IRQ_MASK 0x00000040
    #define IRQ_GPIO_FIQ_MASK 0x00000080
    #define IRQ_AVP_MASK      0x00000100
    #define IRQ_AVE0_MASK     0x00000200
    #define IRQ_AVE1_MASK     0x00000400
    #define IRQ_JPEG_MASK     0x00000800
    #define IRQ_DMA_MASK      0x00001000
    #define IRQ_ETHERNET_MASK 0x00002000
    #define IRQ_USB_MASK      0x00004000
    #define IRQ_HOSTIF_MASK   0x00008000
    #define IRQ_AES_MASK      0x00010000
    #define IRQ_I2C_MASK      0x00020000
    #define IRQ_SSI_MASK      0x00040000
    #define IRQ_COMMRX_MASK   0x00080000
    #define IRQ_COMMTX_MASK   0x00100000
    #define IRQ_FORCE_MASK    0x00200000
    #define IRQ_SDHC0_MASK    0x00400000
    #define IRQ_SDHC1_MASK    0x00800000
    #define IRQ_SMIA_MASK     0x01000000
    #define IRQ_VIDOUT_MASK   0x02000000
    #define IRQ_CPUS_MASK     0x04000000
    #define IRQ_MBDMA_MASK    0x08000000

    #define IRQ_UART0    (0)
    #define IRQ_UART1    (1)
    #define IRQ_TIMER0   (2)
    #define IRQ_TIMER1   (3)
    #define IRQ_TIMER2   (4)
    #define IRQ_TIMER3   (5)
    #define IRQ_GPIO_IRQ (6)
    #define IRQ_GPIO_FIQ (7)
    #define IRQ_AVP      (8)
    #define IRQ_AVE0     (9)
    #define IRQ_AVE1     (10)
    #define IRQ_JPEG     (11)
    #define IRQ_DMA      (12)
    #define IRQ_ETHERNET (13)
    #define IRQ_USB      (14)
    #define IRQ_HOSTIF   (15)
    #define IRQ_AES      (16)
    #define IRQ_I2C      (17)
    #define IRQ_SSI      (18)
    #define IRQ_COMMRX   (19)
    #define IRQ_COMMTX   (20)
    #define IRQ_FORCE    (21)
    #define IRQ_SDHC0    (22)
    #define IRQ_SDHC1    (23)
    #define IRQ_SMIA     (24)
    #define IRQ_VIDOUT   (25)
    #define IRQ_CPUS     (26)
    #define IRQ_MBDMA    (27)
    // Number of IRQ sources (defined in irqs.h)
	//#define NR_IRQS      (28)

  REG32 LEVEL;      //IQLR;              // Level Register 0=>IRQ, 1=>FIQ
  REG32 FIQ_SRCS;   //IQFP;              // Fiq Pending (after mask and level)
  REG32 SRCS;     //IQPR;              // Pending Register (before mask)
} tIRQ;

//-----------------------------
// Timer
//-----------------------------

typedef struct _tTIMER
{
  REG32 MATCH[4];                         // Match register 0-3
  REG32 COUNTER;                          // Free running clock (1MHz)
  REG32 STATUS;                           // Match status flags (bit 0-3)
  REG32 FORCE;                            // Match force regiser (bit 0-3)
} tTIMER;

//-----------------------------
// GPIO Controller
//-----------------------------

typedef struct _tGPIO
{
  #define NUM_GPIO 96    // Number of GPIO pins

  REG32 PIN[NUM_GPIO];
    #define GET_GPIO_MODE(x) (((x)>>0)&0x7)
    #define SET_GPIO_MODE(upd) ((upd)<<0)
      #define GPIO_MODE_IN        0
      #define GPIO_MODE_IN_IRQ    1
      #define GPIO_MODE_IN_FIQ    2
      #define GPIO_MODE_IN_PU     3
      #define GPIO_MODE_IN_PU_IRQ 4
      #define GPIO_MODE_IN_PU_FIQ 5
      #define GPIO_MODE_OUT       6
      #define GPIO_MODE_ALT_FUNC  7

    #define GET_GPIO_LEVEL(x) (((x)>>3)&0x1)
    #define SET_GPIO_LEVEL(upd) ((upd)<<3)

    #define GET_GPIO_TRIG(x) (((x)>>4)&0x1)
    #define SET_GPIO_TRIG(upd) ((upd)<<4)

    #define GET_GPIO_EDP(x) (((x)>>5)&0x1)
    #define SET_GPIO_EDP(upd) ((upd)<<5)

  REG32 __gpio_pad[128-NUM_GPIO];
  REG32 PRIO;
    #define GET_GPIO_PRIO_IRQ_NBR(x) (((x)>>0)&0x7f)
    #define GET_GPIO_PRIO_IRQ(x) (((x)>>7)&0x1)
    #define GET_GPIO_PRIO_FIQ_NBR(x) (((x)>>8)&0x7f)
    #define GET_GPIO_PRIO_FIQ(x) (((x)>>15)&0x1)
} tGPIO;

//-----------------------------
// PIN Controller
//-----------------------------

typedef struct _tPINCTRL
{
  #define NUM_PINCTRL 21    // Number of PINCTRL groups

  REG32 GROUP[NUM_PINCTRL];
    #define GET_PINCTRL_MPIO(x) (((x)>>0)&0x7)
    #define SET_PINCTRL_MPIO(upd) ((upd)<<0)
    #define GET_PINCTRL_DDR(x) (((x)>>3)&0x3)
    #define SET_PINCTRL_DDR(upd) ((upd)<<3)

      #define MODE_DDR_90_OHM 2
      #define MODE_DDR_60_OHM 1
      #define MODE_DDR_30_OHM 3
      #define MODE_DDR_HI_Z   0

      #define MODE_MPIO_70_OHM 0
      #define MODE_MPIO_40_OHM 1
      #define MODE_MPIO_32_OHM 3
      #define MODE_MPIO_20_OHM 7

  REG32 __pinctrl_pad[32-NUM_PINCTRL];
  REG32 CHIPID[2];        // 64 bit chipid
} tPINCTRL;


//-----------------------------
// Pulse Wave Modulators
//-----------------------------

typedef struct _tPWM
{
  REG32 PRESCALE;
    #define PWM_PRESCALE_1   0
    #define PWM_PRESCALE_16  1
    #define PWM_PRESCALE_256 2
    #define PWM_OFF 3
  REG32 FORCE_OUTPUT;
  REG32 PERIOD;
    #define SET_PWM_LO(x) ((x)&0xffff)
    #define GET_PWM_LO(x) ((x)&0xffff)
    #define SET_PWM_HI(x) (((x)&0xffff)<<16)
    #define GET_PWM_HI(x) (((x)>>16)&0xffff)
  REG32 COUNTER;
  REG32 _PAD[(0x10000-4*4)/4];
} tPWM;

//-----------------------------
// Video Processor
//-----------------------------

typedef struct _tAVP
{
  REG32 STATUS;           // 0x0000 : Common status register, clear bits by writing '1'
  REG32 IRQ_MASK;         // 0x0004 : Mask for interrupt generation based on status register
    #define AVP_ALWAYS_MASK (1 << 14) // Always true condition for forcing software interrupt
    #define AVP_FLD1_MASK   (1 << 13) // Field 1 detected on video input interface
    #define AVP_FLD0_MASK   (1 << 12) // Field 0 detected on video input interface
    #define AVP_FST_MASK    (1 << 11) // Frame sync is received on video input interface
    #define AVP_FEND_MASK   (1 << 10) // Whole VCUT image has been delivered through video input interface
    #define AVP_FERR_MASK   (1 <<  9) // Unexpected external frame sync (within frame or when input is disabled)
    #define AVP_RASTER_MASK (1 <<  8) // Set when video interface line counter matches RASTER register
    #define AVP_DONE_MASK   (1 <<  7) // Full image has been processed by image processing
    #define AVP_RATE_MASK   (1 <<  6) // Too high pixel frequency or too short horizontal blanking
    #define AVP_OF5_MASK    (1 <<  5) // Overflow has occurred in DMA channel 5 (analyze output)
    #define AVP_OF4_MASK    (1 <<  4) // Overflow has occurred in DMA channel 4 (scaler 1 UV output)
    #define AVP_OF3_MASK    (1 <<  3) // Overflow has occurred in DMA channel 3 (scaler 1 Y output)
    #define AVP_OF2_MASK    (1 <<  2) // Overflow has occurred in DMA channel 2 (scaler 0 UV output)
    #define AVP_OF1_MASK    (1 <<  1) // Overflow has occurred in DMA channel 1 (scaler 0 Y output)
    #define AVP_OF0_MASK    (1 <<  0) // Overflow has occurred in DMA channel 0 (bayer output)

  REG32 DMA_IN;           // 0x0008: Format of DMA input stream for image processing
    #define SET_AVP_DMA_IN_YSIZE(x)    (((x)& 0x3fff)<<16)  // height in pixels - 1
    #define SET_AVP_DMA_IN_XSIZE(x)    (((x)& 0x3fff)<< 2)  // width in pixels - 1
    #define SET_AVP_DMA_IN_BITS(x)     (((x)&    0x3)<< 0)  // Bits per value: 0=8-bit, 1=10-bit, 2=12-bit

  REG32 DECOMPRESS[3];    // 0x000c: Linearization
    #define SET_AVP_DECOMPRESS_EXP(x)  (((x)&   0x7)<<22)  // Amplification exponent (0..7)
    #define SET_AVP_DECOMPRESS_Y(x)    (((x)&0x3fff)<< 8)  // Output knee offset (0..16383)*16
    #define SET_AVP_DECOMPRESS_X(x)    (((x)&  0xff)<< 0)  // Input knee offset  (0..255)*16

  REG32 GAIN;             // 0x0018: Digital gain
    #define SET_AVP_GAIN_BEXP(x)       (((x)&   0x7)<<21)  // Blue channel gain, exponent part (-4..3)
    #define SET_AVP_GAIN_BFRAC(x)      (((x)&  0x1f)<<16)  // Blue channel gain, fraction part (0..31)
    #define SET_AVP_GAIN_GEXP(x)       (((x)&   0x7)<<13)  // Green channel gain, exponent part (-4..3)
    #define SET_AVP_GAIN_GFRAC(x)      (((x)&  0x1f)<< 8)  // Green channel gain, fraction part (0..31)
    #define SET_AVP_GAIN_REXP(x)       (((x)&   0x7)<< 5)  // Red channel gain, exponent part (-4..3)
    #define SET_AVP_GAIN_RFRAC(x)      (((x)&  0x1f)<< 0)  // Red channel gain, fraction part (0..31)

  REG32 COMPRESS;         // 0x001c: Compression above a certain threshold
    #define SET_AVP_COMPRESS_EXP(x)    (((x)&   0x7)<< 8)  // Attenuation exponent 0..7
    #define SET_AVP_COMPRESS_X(x)      (((x)&  0xff)<< 0)  // End of linear range (0..255)*4

  REG32 DPC;              // 0x0020: Defect pixel correction
    #define SET_AVP_DPC_H(x)           (((x)&  0x3)<<11)  // Correction hardness
    #define SET_AVP_DPC_K(x)           (((x)&  0x3)<< 9)  // Amplitude coefficient
    #define SET_AVP_DPC_D(x)           (((x)& 0xff)<< 1)  // Defect offset
    #define SET_AVP_DPC_STATIC(x)      (((x)&  0x1)<< 0)  // Enable static DPC list

  REG32 HCUT;             // 0x0024: Horizontal cut of image processing input stream
    #define SET_AVP_HCUT_XSTART(x)     (((x)& 0x3fff)<<10)  // No. of pixels to skip
    #define SET_AVP_HCUT_XSIZE(x)      (((x)&  0x3ff)<< 0)  // width in pixels / 4
    #define GET_AVP_HCUT_XSTART(x)     (((x)>>10)& 0x3fff)
    #define GET_AVP_HCUT_XSIZE(x)      (((x)>> 0)&  0x3ff)

  REG32 DEMOSAIC;         // 0x0028: Bayer to RGB interpolation control, YUV decoding
    #define SET_AVP_DEMOSAIC_LE(x)     (((x)& 0x1) << 15)  // 1 = Discard 2 output pixels at left edge
    #define SET_AVP_DEMOSAIC_RE(x)     (((x)& 0x1) << 14)  // 1 = Discard 2 output pixels at right edge
    #define SET_AVP_DEMOSAIC_TE(x)     (((x)& 0x1) << 13)  // 1 = Discard 2 output pixels at top edge
    #define SET_AVP_DEMOSAIC_BE(x)     (((x)& 0x1) << 12)  // 1 = Discard 2 output pixels at bottom edge
    #define SET_AVP_DEMOSAIC_DY(x)     (((x)& 0x1) << 11)  // 0 = First line is G1/R, 1 = first line is B/G2
    #define SET_AVP_DEMOSAIC_DX(x)     (((x)& 0x1) << 10)  // 0 = First column is G1/B, 1 = first column is R/G2
    #define SET_AVP_DEMOSAIC_V(x)      (((x)& 0x3) <<  8)  // ordering of V in YYUV input stream
    #define SET_AVP_DEMOSAIC_U(x)      (((x)& 0x3) <<  6)  // ordering of U in YYUV input stream
    #define SET_AVP_DEMOSAIC_Y1(x)     (((x)& 0x3) <<  4)  // ordering of Y1 in YYUV input stream
    #define SET_AVP_DEMOSAIC_Y0(x)     (((x)& 0x3) <<  2)  // ordering of Y0 in YYUV input stream
    #define SET_AVP_DEMOSAIC_FORMAT(x) (((x)& 0x3) <<  0)  // 0=bayer 1=YY/UV 2=YYUV
    #define GET_AVP_DEMOSAIC_LE(x)     (((x) >> 15) & 0x1)
    #define GET_AVP_DEMOSAIC_RE(x)     (((x) >> 14) & 0x1)
    #define GET_AVP_DEMOSAIC_TE(x)     (((x) >> 13) & 0x1)
    #define GET_AVP_DEMOSAIC_BE(x)     (((x) >> 12) & 0x1)
    #define GET_AVP_DEMOSAIC_DY(x)     (((x) >> 11) & 0x1)
    #define GET_AVP_DEMOSAIC_DX(x)     (((x) >> 10) & 0x1)
    #define GET_AVP_DEMOSAIC_V(x)      (((x) >>  8) & 0x3)
    #define GET_AVP_DEMOSAIC_U(x)      (((x) >>  6) & 0x3)
    #define GET_AVP_DEMOSAIC_Y1(x)     (((x) >>  4) & 0x3)
    #define GET_AVP_DEMOSAIC_Y0(x)     (((x) >>  2) & 0x3)
    #define GET_AVP_DEMOSAIC_FORMAT(x) (((x) >>  0) & 0x3)

  REG32 BIAS;             // 0x002c: Filtering of G1/G2 bias artifacts, digital red/blue) gain
    #define SET_AVP_BIAS_BGAIN(x)    (((x)& 0xf)  << 28) // Digital gain of blue channel
                                                         // [-8 .. -1] maps to factor [0.5 .. 0.9375 ]
                                                         // [ 0 ..  7] maps to factor [1.0 .. 1.875  ]
    #define SET_AVP_BIAS_RGAIN(x)    (((x)& 0xf)  << 24) // Digital gain of red channel (see BGAIN)
    #define SET_AVP_BIAS_THRESH(x)   (((x)& 0xff) << 16) // Threshold for dynamic G1/G2 adjustment (unsigned)
    #define SET_AVP_BIAS_G2(x)       (((x)& 0xff) <<  8) // Static G2 adjustment (signed)
    #define SET_AVP_BIAS_G1(x)       (((x)& 0xff) <<  0) // Static G1 adjustment (signed)
    #define GET_AVP_BIAS_BGAIN(x)    (((x) >> 28)& 0xf)
    #define GET_AVP_BIAS_RGAIN(x)    (((x) >> 24)& 0xf)
    #define GET_AVP_BIAS_THRESH(x)   (((x) >> 16)& 0xff)
    #define GET_AVP_BIAS_G2(x)       (((x) >>  8)& 0xff)
    #define GET_AVP_BIAS_G1(x)       (((x) >>  0)& 0xff)

  REG32 ANALYZE;          // 0x0030: Block-based statistics and histogram control
    #define SET_AVP_ANALYZE_SHIFT(x)  (((x)& 0x7) <<10)  // Downshift of sum in output (except when MINMAX=1)
    #define SET_AVP_ANALYZE_SIZEY(x)  (((x)& 0x7) << 7)  // Block height (0=8, 1=16, 2=24, 3=32 .. 7=64)
    #define SET_AVP_ANALYZE_SIZEX(x)  (((x)& 0x7) << 4)  // Block width  (0=8, 1=16, 2=24, 3=32 .. 7=64)
    #define SET_AVP_ANALYZE_COMPACT(x)(((x)& 0x1) << 3)  // Enable compact max/sum format in block-based statistics
    #define SET_AVP_ANALYZE_MINMAX(x) (((x)& 0x1) << 2)  // Enable min/max values in block-based statistics
    #define SET_AVP_ANALYZE_BLOCK(x)  (((x)& 0x1) << 1)  // Enable block-based statistics
    #define SET_AVP_ANALYZE_HIST(x)   (((x)& 0x1) << 0)  // Enable histogram

  REG32 AFWINDOW;         // 0x0034: Auto-focus window (relative to HCUT/VCUT window)
    #define SET_AVP_AFWINDOW_Y1(x)   (((x)& 0xff) << 24) // End Y coordinate/16
    #define SET_AVP_AFWINDOW_Y0(x)   (((x)& 0xff) << 16) // Starting Y coordinate/16
    #define SET_AVP_AFWINDOW_X1(x)   (((x)& 0xff) <<  8) // End X coordinate/16
    #define SET_AVP_AFWINDOW_X0(x)   (((x)& 0xff) <<  0) // Starting X coordinate/16
    #define GET_AVP_AFWINDOW_Y1(x)   (((x) >> 24)& 0xff)
    #define GET_AVP_AFWINDOW_Y0(x)   (((x) >> 16)& 0xff)
    #define GET_AVP_AFWINDOW_X1(x)   (((x) >>  8)& 0xff)
    #define GET_AVP_AFWINDOW_X0(x)   (((x) >>  0)& 0xff)

  REG32 AFRESULT;         // 0x0038: Auto-focus analysis result, 32-bit unsigned

  REG32 SHARP;            // 0x003c: Sharpening filter configuration
    #define SET_AVP_SHARP_K3(x)     (((x)& 0x3f) << 26)  // Coefficient for G2 pixels
    #define SET_AVP_SHARP_K2(x)     (((x)& 0x3f) << 20)  // Coefficient for B pixels
    #define SET_AVP_SHARP_K1(x)     (((x)& 0x3f) << 14)  // Coefficient for R pixels
    #define SET_AVP_SHARP_K0(x)     (((x)& 0x3f) <<  8)  // Coefficient for G1 pixels
    #define SET_AVP_SHARP_THRESH(x) (((x)& 0xff) <<  0)  // Threshold for sharpening term
    #define GET_AVP_SHARP_K3(x)     (((x)>> 26) & 0x3f)
    #define GET_AVP_SHARP_K2(x)     (((x)>> 20) & 0x3f)
    #define GET_AVP_SHARP_K1(x)     (((x)>> 14) & 0x3f)
    #define GET_AVP_SHARP_K0(x)     (((x)>>  8) & 0x3f)
    #define GET_AVP_SHARP_THRESH(x) (((x)>>  0) & 0xff)

  REG32 RADIAL_X0;        // 0x0040: Location of first input pixel with respect to optical axis, 14-bit signed
  REG32 RADIAL_Y0;        // 0x0044: Location of first input pixel with respect to optical axis, 14-bit signed
  REG32 RADIAL_X02;       // 0x0048: Square of RADIAL_X0, 26-bit unsigned
  REG32 RADIAL_Y02;       // 0x004c: Square of RADIAL_Y0, 26-bit unsigned
  REG32 RADIAL_SHIFT;     // 0x0050: Exponent for radial illumination correction, 3-bit unsigned
  REG32 RADIAL_COEFF[6];  // 0x0054: Radial corrrection coefficents, 6-bit unsigned
                          //         Order is kxr kyr kxg kyg kxb kyb

  REG32 CLIP;             // 0x006c: Truncation of RGB values before color correction
    #define SET_AVP_CLIP_BMAX(x) (((x)& 0x3ff) << 20)  // Upper limit for blue channel
    #define SET_AVP_CLIP_GMAX(x) (((x)& 0x3ff) << 10)  // Upper limit for green channel
    #define SET_AVP_CLIP_RMAX(x) (((x)& 0x3ff) <<  0)  // Upper limit for red channel

  REG32 COLOR_CORR[12];   // 0x0070: Color correction coefficients, signed 14-bit
                          //         Order is C11, C12, C13, C21, C22, C23, C21, C22, C23, D1, D2, D3

  REG32 GAMMA;            // 0x00a0: Gamma correction enable bit, 1=enabled

  REG32 COLOR_CONV[12];   // 0x00a4: Color conversion coefficients, signed 9-bit
                          //         Order is C11, C12, C13, C21, C22, C23, C21, C22, C23, D1, D2, D3

  REG32 HSCALE[2];        // 0x00d4: Horizontal downscale (2 units)
    #define SET_AVP_HSCALE_SIZE(x)  (((x)&0xfff) << 20)  // Output width in pixels - 1
    #define SET_AVP_HSCALE_EXP(x)   (((x)& 0x0f) << 16)  // Denominator exponent part (-2..7)
    #define SET_AVP_HSCALE_FRAC(x)  (((x)& 0xff) <<  8)  // Denominator fraction part
    #define SET_AVP_HSCALE_FPOS(x)  (((x)& 0xff) <<  0)  // Initial sub-pixel offset
    #define GET_AVP_HSCALE_SIZE(x)  (((x) >> 20) & 0xfff)
    #define GET_AVP_HSCALE_EXP(x)   (((x) >> 16) & 0x0f)
    #define GET_AVP_HSCALE_FRAC(x)  (((x) >>  8) & 0xff)
    #define GET_AVP_HSCALE_FPOS(x)  (((x) >>  0) & 0xff)

  REG32 VSCALE[2];        // 0x00dc: Vertical downscale (2 units)
    #define SET_AVP_VSCALE_SIZE(x)  (((x)&0xfff) << 20)  // Output width in pixels - 1
    #define SET_AVP_VSCALE_EXP(x)   (((x)& 0x0f) << 16)  // Denominator exponent part (-2..7)
    #define SET_AVP_VSCALE_FRAC(x)  (((x)& 0xff) <<  8)  // Denominator fraction part
    #define SET_AVP_VSCALE_FPOS(x)  (((x)& 0xff) <<  0)  // Initial sub-pixel offset
    #define GET_AVP_VSCALE_SIZE(x)  (((x) >> 20) & 0xfff)
    #define GET_AVP_VSCALE_EXP(x)   (((x) >> 16) & 0x0f)
    #define GET_AVP_VSCALE_FRAC(x)  (((x) >>  8) & 0xff)
    #define GET_AVP_VSCALE_FPOS(x)  (((x) >>  0) & 0xff)

  REG32 OUTPUT[2];        // 0x00e4: Format of DMA image output streams, YUV encoding (2 units)
    #define SET_AVP_OUTPUT_RMODE(x)  (((x)& 0x07) << 9)  // Rounding mode 0-7
    #define SET_AVP_OUTPUT_CVDIV(x)  (((x)& 0x01) << 8)  // Extra factor 2 chroma horizontal downscale
    #define SET_AVP_OUTPUT_CHDIV(x)  (((x)& 0x01) << 7)  // Extra factor 2 chroma vertical downscale
    #define SET_AVP_OUTPUT_RGB(x)    (((x)& 0x01) << 6)  // Bypass YUV conversion
    #define SET_AVP_OUTPUT_FORMAT(x) (((x)& 0x03) << 4)  // Encoding: 0=YY+U+V 1=YY+UV 2=YYUV 3=YUYV
    #define SET_AVP_OUTPUT_10BIT(x)  (((x)& 0x01) << 3)  // Bits per value: 0=8-bit, 1=10-bit
    #define SET_AVP_OUTPUT_V(x)      (((x)& 0x01) << 2)  // Enable V (ternary) output channel (DMA3)
    #define SET_AVP_OUTPUT_U(x)      (((x)& 0x01) << 1)  // Enable U (second) output channel  (DMA2)
    #define SET_AVP_OUTPUT_Y(x)      (((x)& 0x01) << 0)  // Enable Y (primary) output channel (DMA1)

  REG32 FIFO_S0_C;        // 0x00ec: Base of scaler 0 chroma FIFO, 12 bits unsigned
  REG32 FIFO_S1_Y;        // 0x00f0: Base of scaler 1 luminance FIFO, 12 bits unsigned
  REG32 FIFO_S1_C;        // 0x00f4: Base of scaler 1 chroma FIFO, 12 bits unsigned

  REG32 IMAGE_CTRL;       // 0x00f8: Main control register for image processing
    #define SET_AVP_IMAGE_CTRL_NOSLEEP(x)  (((x)& 0x01) << 2) // 1= disable clock sleep during blanking
    #define SET_AVP_IMAGE_CTRL_DMA_MODE(x) (((x)& 0x01) << 1) // Input selection: 0=sensor 1=dma
    #define SET_AVP_IMAGE_CTRL_ENABLE(x)   (((x)& 0x01) << 0) // Enable image processing

  REG32 VCUT;             // 0x00fc: Vertical cut of video input image. Height is YEND-YSTART+1 pixels
    #define SET_AVP_VCUT_YSTART(x)  (((x)& 0x3fff) << 14) // First line to include
    #define SET_AVP_VCUT_YEND(x)    (((x)& 0x3fff) <<  0) // Last line to include. Height is YEND-YSTART+1 pixels
    #define GET_AVP_VCUT_YSTART(x)  (((x) >> 14) & 0x3fff)
    #define GET_AVP_VCUT_YEND(x)    (((x) >> 0 ) & 0x3fff)

  REG32 BAYER_OUT;        // 0x0100: DMA output control from video input interface, horizontal cut
    #define SET_AVP_BAYER_OUT_XSTART(x) (((x)& 0x3fff) << 17) // First column to include
    #define SET_AVP_BAYER_OUT_XEND(x)   (((x)& 0x3fff) <<  3) // Last column to include. Width is XEND-XSTART+1 pixels
    #define SET_AVP_BAYER_OUT_BITS(x)   (((x)&  0x03) <<  1) // Bits per value: 0=8-bit, 1=10-bit, 2=12-bit
    #define SET_AVP_BAYER_OUT_ENABLE(x) (((x)&  0x01) <<  0) // Enable bayer output channel (DMA0)
    #define GET_AVP_BAYER_OUT_XSTART(x) (((x) >> 17) & 0x3fff)
    #define GET_AVP_BAYER_OUT_XEND(x)   (((x) >>  3) & 0x3fff)
    #define GET_AVP_BAYER_OUT_BITS(x)   (((x) >>  1) &   0x03)
    #define GET_AVP_BAYER_OUT_ENABLE(x) (((x) >>  0) &   0x01)

  REG32 RASTER;           // 0x0104: Vertical position for raster interrupt, 14-bit unsigned

  REG32 VIDIN_CTRL;       // 0x0108: Main control register for video input interface
    #define SET_AVP_VIDIN_CTRL_RMODE(x)    (((x)&   0x7) << 26) // Rounding mode 0-7
    #define SET_AVP_VIDIN_CTRL_FORCE_CLK(x)(((x)&   0x1) << 25) // 1=force internal clock to be enabled
    #define SET_AVP_VIDIN_CTRL_CSUB(x)     (((x)&   0x3) << 23) // CCIR specific subsampling factor, 0=off
    #define SET_AVP_VIDIN_CTRL_IMSIZE_X(x) (((x)& 0x3fff) << 9) // Width of input image (no of pixels - 1)
    #define SET_AVP_VIDIN_CTRL_YSUB(x)     (((x)&   0x01) << 8) // 1=enable vertical subsampling
    #define SET_AVP_VIDIN_CTRL_XSUB(x)     (((x)&   0x01) << 7) // 1=enable horizontal subsampling
    #define SET_AVP_VIDIN_CTRL_CCIR(x)     (((x)&   0x01) << 6) // 1=enable CCIR-656 decoding
    #define SET_AVP_VIDIN_CTRL_BITS(x)     (((x)&   0x03) << 4) // 0=8-bit VAD, 1=10-bit VAD, 2=12-bit VAD
    #define SET_AVP_VIDIN_CTRL_FST_EDGE(x) (((x)&   0x01) << 3) // 0=negative VFST edge, 1=positive
    #define SET_AVP_VIDIN_CTRL_CLK_EDGE(x) (((x)&   0x01) << 2) // 0=negative VCLK edge, 1=positive
    #define SET_AVP_VIDIN_CTRL_PIX_VAL(x)  (((x)&   0x01) << 1) // 0=VPIX active low, 1=active high
    #define SET_AVP_VIDIN_CTRL_ENABLE(x)   (((x)&   0x01) << 0) // Enable video input
    #define GET_AVP_VIDIN_CTRL_RMODE(x)    (((x) >> 26)&   0x7)
    #define GET_AVP_VIDIN_CTRL_FORCE_CLK(x)(((x) >> 25)&   0x1)
    #define GET_AVP_VIDIN_CTRL_CSUB(x)     (((x) >> 23)&   0x3)
    #define GET_AVP_VIDIN_CTRL_IMSIZE_X(x) (((x) >> 9)& 0x3fff)
    #define GET_AVP_VIDIN_CTRL_YSUB(x)     (((x) >> 8)&   0x01)
    #define GET_AVP_VIDIN_CTRL_XSUB(x)     (((x) >> 7)&   0x01)
    #define GET_AVP_VIDIN_CTRL_CCIR(x)     (((x) >> 6)&   0x01)
    #define GET_AVP_VIDIN_CTRL_BITS(x)     (((x) >> 4)&   0x03)
    #define GET_AVP_VIDIN_CTRL_FST_EDGE(x) (((x) >> 3)&   0x01)
    #define GET_AVP_VIDIN_CTRL_CLK_EDGE(x) (((x) >> 2)&   0x01)
    #define GET_AVP_VIDIN_CTRL_PIX_VAL(x)  (((x) >> 1)&   0x01)
    #define GET_AVP_VIDIN_CTRL_ENABLE(x)   (((x) >> 0)&   0x01)

  REG32 LED1;             // 0x010c: LED strobe timing and active level
    #define SET_LED1_ON_PERIOD(x)    (((x)& 0xfff) << 13) // Duration of LED strobe
    #define SET_LED1_ON_TIME(x)      (((x)& 0xfff) <<  1) // Delay from frame sync to LED activation
    #define SET_LED1_ACTIVE_LEVEL(x) (((x)&  0x01) <<  0) // 0=active low, 1=active high

  REG32 LED2;             // 0x0110: LED strobe clock prescale and enable bit
    #define SET_LED2_PRESCALE(x)     (((x)& 0xfff) <<  1) // Clock pre-scale factor for LED timing
    #define SET_LED2_ENABLE(x)       (((x)&  0x01) <<  0) // 0=LED disabled, 1=enabled

  REG32 pad0[1024-69];
  REG32 GAMMA_R[64];      // 0x1000: Gamma table for R channel SLOPE[7:0] START[9:0]
  REG32 GAMMA_G[64];      // 0x1100: Gamma table for G channel
  REG32 GAMMA_B[64];      // 0x1200: Gamma table for B channel
  REG32 pad1[1024-192];

  REG32 SCALE_RAM[2*3072]; //0x2000: Scale RAM 3072*(2*20)  (low0, high0, low1, high1, ...)

  REG32 BAYER_RAM[2*2592]; //0x8000: Bayer and analysis temp storage 2752*(2*20) (low0, high0, low1, high1, ...)
  REG32 HIST_G1[64];
  REG32 HIST_R[64];
  REG32 HIST_B[64];
  REG32 HIST_G2[64];
  REG32 DPC_LIST[2*32];

} tAVP;

//-----------------------------
// Video Output
//-----------------------------

typedef struct _tVIDOUT
{
  REG32 STATUS;          // 0x0000 : Common status register, clear bits by writing '1'     
  REG32 IRQ_MASK;        // 0x0004 : Mask for interrupt generation based on status register
    #define VIDOUT_ALWAYS_MASK  (1 << 5)  // Always active event
    #define VIDOUT_FST1_MASK    (1 << 4)  // Active video has started, field 1
    #define VIDOUT_FST0_MASK    (1 << 3)  // Active video has started, field 0
    #define VIDOUT_FEND_MASK    (1 << 2)  // End of active video has been reached
    #define VIDOUT_RASTER_MASK  (1 << 1)  // Raster interrupt
    #define VIDOUT_UR_MASK      (1 << 0)  // Underrun

  REG32 HPERIOD;         // 0x0008: Line duration in VOCLK cycles
  REG32 VPERIOD;         // 0x000c: Frame duration in no. of lines
  REG32 HACTIVE;         // 0x0010: No. of active video VOCLK cycles
  REG32 VACTIVE;         // 0x0014: No. of active video lines
  REG32 HSYNC[2];        // 0x0018: 12 bits: [0]=cycle where HSYNC goes low [1]=cycle where HSYNC goes high
  REG32 VSYNC[2];        // 0x0020: 12 bits: [0]=line where VSYNC goes low  [1]=line where VSYNC goes high
                         //          plus some additional bits for more exact placement
    #define SET_VSYNC_LINE(x)       (((x)<<0))
    #define GET_VSYNC_LINE(x)       (((x)>>0)&0xfff)
    #define SET_VSYNC_HPOS(x)       (((x)<<12))     // 0=start of HACTIVE 1=on falling HSYNC 2=on rising HSYNC
    #define GET_VSYNC_HPOS(x)       (((x)>>12)&0x3)

  REG32 VPOS;            // 0x0028: Vertical raster counter, 12-bit unsigned + field no.
    #define SET_VIDOUT_VPOS_FIELD(x) (((x)<<12))
    #define GET_VIDOUT_VPOS_FIELD(x) (((x)>>12)&1)
    #define SET_VIDOUT_VPOS_Y(x)     (((x)<<0))
    #define GET_VIDOUT_VPOS_Y(x)     (((x)>>0)&0xfff)

  REG32 RASTER;          // 0x002c: Vertical position for raster interrupt, 12-bit unsigned

  REG32 INTERLACE;       // 0x0030: Interlaced mode control
    #define SET_VIDOUT_INTERLACE_FLINE(x)   (((x)<<3))    // Line where CCIR F bit is toggled
    #define GET_VIDOUT_INTERLACE_FLINE(x)   (((x)>>3)&0xfff)
    #define SET_VIDOUT_INTERLACE_PERIOD(x)  (((x)<<2))    // 1=make field 0 one line longer
    #define GET_VIDOUT_INTERLACE_PERIOD(x)  (((x)>>2)&1)
    #define SET_VIDOUT_INTERLACE_ACTIVE(x)  (((x)<<1))    // 1=make field 0 active video one line longer
    #define GET_VIDOUT_INTERLACE_ACTIVE(x)  (((x)>>1)&1) 
    #define SET_VIDOUT_INTERLACE_ENABLE(x)  (((x)<<0))    // 1=Enable field alternation
    #define GET_VIDOUT_INTERLACE_ENABLE(x)  (((x)>>0)&1)

  REG32 CTRL;            // 0x0034: Control register
    #define SET_VIDOUT_CTRL_TEST(x)      (((x)<<12))    // Output test pattern
    #define GET_VIDOUT_CTRL_TEST(x)      (((x)>>12)&1)
    #define SET_VIDOUT_CTRL_DATA12(x)    (((x)<<11))    // 1=Use both VODATA[7:0] and VAD[3:0] for video output
    #define GET_VIDOUT_CTRL_DATA12(x)    (((x)>>11)&1)
    #define SET_VIDOUT_CTRL_STUFF(x)     (((x)<<10))    // Stuff RGB as 2x12 bits
    #define GET_VIDOUT_CTRL_STUFF(x)     (((x)>>10)&1)
    #define SET_VIDOUT_CTRL_RANGE(x)     (((x)<<9))     // Output range 1=[0..ff] 1=[10..ef]
    #define GET_VIDOUT_CTRL_RANGE(x)     (((x)>>9)&1)
    #define SET_VIDOUT_CTRL_UPSAMPLE(x)  (((x)<<7))     // 0=fullres 1=halfres 2=one 3rd res. 3=quarter res.
    #define GET_VIDOUT_CTRL_UPSAMPLE(x)  (((x)>>7)&3)
    #define SET_VIDOUT_CTRL_DDR(x)       (((x)<<6))     // Enable double data rate (two-edge clocked)
    #define GET_VIDOUT_CTRL_DDR(x)       (((x)>>6)&1)
    #define SET_VIDOUT_CTRL_CCIR2(x)     (((x)<<5))     // Output double-length sync words
    #define GET_VIDOUT_CTRL_CCIR2(x)     (((x)>>5)&1)
    #define SET_VIDOUT_CTRL_CCIR(x)      (((x)<<4))     // Enable CCIR-656 embedded sync words
    #define GET_VIDOUT_CTRL_CCIR(x)      (((x)>>4)&1)
    #define SET_VIDOUT_CTRL_CLK_DIV(x)   (((x)<<3))     // 1=Divide clock by 2 before use
    #define GET_VIDOUT_CTRL_CLK_DIV(x)   (((x)>>3)&1)
    #define SET_VIDOUT_CTRL_CLK_EDGE(x)  (((x)<<2))     // 0=negative edge 1=positive edge
    #define GET_VIDOUT_CTRL_CLK_EDGE(x)  (((x)>>2)&1)
    #define SET_VIDOUT_CTRL_CLK_OUT(x)   (((x)<<1))     // Enable driver to use VOCLK as output
    #define GET_VIDOUT_CTRL_CLK_OUT(x)   (((x)>>1)&1) 
    #define SET_VIDOUT_CTRL_ENABLE(x)    (((x)<<0))
    #define GET_VIDOUT_CTRL_ENABLE(x)    (((x)>>0)&1)
  
  REG32 BORDER;          // 0x0038: Border color + default alpha value
    #define SET_VIDOUT_BORDER_ALPHA(x)   (((x)<<24))       // Default value for alpha channel
    #define GET_VIDOUT_BORDER_ALPHA(x)   (((x)>>24)&0xff)
    #define SET_VIDOUT_BORDER_V(x)       (((x)<<16))       // Border Y (R when blending RGB)
    #define GET_VIDOUT_BORDER_V(x)       (((x)>>16)&0xff)
    #define SET_VIDOUT_BORDER_U(x)       (((x)<<8))        // Border U (G when blending RGB)
    #define GET_VIDOUT_BORDER_U(x)       (((x)>>8)&0xff)
    #define SET_VIDOUT_BORDER_Y(x)       (((x)<<0))        // Border V (B when blending RGB) 
    #define GET_VIDOUT_BORDER_Y(x)       (((x)>>0)&0xff)

  REG32 YUV_START;       // 0x003c: Top-left corner of YUV image
    #define SET_VIDOUT_YUV_START_Y(x) (((x)<<12))
    #define GET_VIDOUT_YUV_START_Y(x) (((x)>>12)&0xfff)
    #define SET_VIDOUT_YUV_START_X(x) (((x)<<0))
    #define GET_VIDOUT_YUV_START_X(x) (((x)>>0)&0xfff)

  REG32 YUV_END;         // 0x0040: Bottom-right corner of YUV image (+1)
    #define SET_VIDOUT_YUV_END_Y(x) (((x)<<12))
    #define GET_VIDOUT_YUV_END_Y(x) (((x)>>12)&0xfff)
    #define SET_VIDOUT_YUV_END_X(x) (((x)<<0))
    #define GET_VIDOUT_YUV_END_X(x) (((x)>>0)&0xfff)

  REG32 RGBA_START;      // 0x0044: Top-left corner of RGBA image
    #define SET_VIDOUT_RGBA_START_Y(x) (((x)<<12))
    #define GET_VIDOUT_RGBA_START_Y(x) (((x)>>12)&0xfff)
    #define SET_VIDOUT_RGBA_START_X(x) (((x)<<0))
    #define GET_VIDOUT_RGBA_START_X(x) (((x)>>0)&0xfff)

  REG32 RGBA_END;       // 0x0048: Bottom-right corner of YUV image (+1)
    #define SET_VIDOUT_RGBA_END_Y(x) (((x)<<12))
    #define GET_VIDOUT_RGBA_END_Y(x) (((x)>>12)&0xfff)
    #define SET_VIDOUT_RGBA_END_X(x) (((x)<<0))
    #define GET_VIDOUT_RGBA_END_X(x) (((x)>>0)&0xfff)

  REG32 RGB_MATRIX[12]; // 0x004c: YUV to RGB conversion coefficients, signed 10-bit
                        //         Order is C11, C12, C13, C21, C22, C23, C21, C22, C23, D1, D2, D3

  REG32 YUV_MATRIX[12]; // 0x007c: RGB to YUV conversion coefficients, signed 9-bit
                        //         Order is C11, C12, C13, C21, C22, C23, C21, C22, C23, D1, D2, D3

  REG32 IMAGE_CTRL;     // 0x00ac: Image control
    #define SET_VIDOUT_IMAGE_CTRL_RGBA_SWAP(x)   (((x)<<23))   // 1=Swap pixel pairs in DMA2 input
    #define GET_VIDOUT_IMAGE_CTRL_RGBA_SWAP(x)   (((x)>>23)&1)
    #define SET_VIDOUT_IMAGE_CTRL_YUV_SWAP(x)    (((x)<<22))   // 1=Swap pixel pairs in DMA0, DMA1 input
    #define GET_VIDOUT_IMAGE_CTRL_YUV_SWAP(x)    (((x)>>22)&1)
    #define SET_VIDOUT_IMAGE_CTRL_CONV_YUV(x)    (((x)<<21))   // 1=Convert blender output to YUV
    #define GET_VIDOUT_IMAGE_CTRL_CONV_YUV(x)    (((x)>>21)&1)
    #define SET_VIDOUT_IMAGE_CTRL_CONV_RGB(x)    (((x)<<20))   // 1=Convert DMA0/DMA1 to RGB before blending
    #define GET_VIDOUT_IMAGE_CTRL_CONV_RGB(x)    (((x)>>20)&1)
    #define SET_VIDOUT_IMAGE_CTRL_WIDTH(x)       (((x)<<8))    // Pixel width (-1) corresponding to HACTIVE
    #define GET_VIDOUT_IMAGE_CTRL_WIDTH(x)       (((x)>>8)&0xfff)
    #define SET_VIDOUT_IMAGE_CTRL_BPP(x)         (((x)<<6))    // Bytes per pixel: 0=mono 1=yuv 2=rgb
    #define GET_VIDOUT_IMAGE_CTRL_BPP(x)         (((x)>>6)&3)
    #define SET_VIDOUT_IMAGE_CTRL_RGBA_FORMAT(x) (((x)<<5))    // RGBA channel format. 0=YUV/RGB 1=RGBA
    #define GET_VIDOUT_IMAGE_CTRL_RGBA_FORMAT(x) (((x)>>5)&1)
    #define SET_VIDOUT_IMAGE_CTRL_RGBA_4BIT(x) (((x)<<4))    // 1=Use 4-bit encoding for RGBA
    #define GET_VIDOUT_IMAGE_CTRL_RGBA_4BIT(x) (((x)>>4)&1)
    #define SET_VIDOUT_IMAGE_CTRL_YUV_4BIT(x)  (((x)<<3))    // 1=Use 4-bit encoding for YUV
    #define GET_VIDOUT_IMAGE_CTRL_YUV_4BIT(x)  (((x)>>3)&1)
    #define SET_VIDOUT_IMAGE_CTRL_RGBA(x)      (((x)<<2))    // Enable RGBA input channel (DMA2)
    #define GET_VIDOUT_IMAGE_CTRL_RGBA(x)      (((x)>>2)&1)   
    #define SET_VIDOUT_IMAGE_CTRL_UV(x)        (((x)<<1))    // Enable UV input channel (DMA1) 
    #define GET_VIDOUT_IMAGE_CTRL_UV(x)        (((x)>>1)&1)  
    #define SET_VIDOUT_IMAGE_CTRL_Y(x)         (((x)<<0))    // Enable Y input channel (DMA0)
    #define GET_VIDOUT_IMAGE_CTRL_Y(x)         (((x)>>0)&1)
} tVIDOUT;

//-----------------------------
// DMA controller
//-----------------------------

typedef struct _tDMA
{
  struct tDMA_CHANNEL
  {
    REG32 CHACONFIG;
    REG32 CHACTRL;
    REG32 CHASTATUS;
    REG32 __pad0;
    REG32 MDEVADDR;
    REG32 MDEVCONFIG;
    REG32 __pad1[2];
    struct tDMA_DESCQUEUE
    {
  	  REG32 DESC0;
  	  REG32 DESC1;
  	  REG32 DESC2;
  	  REG32 DESC3;
    } DESCQUEUE[2];
  } CHANNEL[10];
  REG32 __pad3[(16-10)*64/4];
  REG32 IRQ;
  REG32 COMMON;
  REG32 __pad4[(4096-16*64-8)/4];
  REG32 RAM[1536/4];
  REG32 __pad5[(65536-4096-1536)/4];
} tDMA;

// CHACONFIG read/write configuration register, normally static. Not changed by the hardware.
#define SET_DMA_CHACONFIG_PORT(x)      ((x)<<0)     // 6  device port select 0-31
#define SET_DMA_CHACONFIG_PRIO(x)      ((x)<<6)     // 2  priority group, 0: lowest prio, 3:highest prio, round robin policy in group 0
#define SET_DMA_CHACONFIG_SIZE(x)      ((x)<<8)     // 3  device size, 0:8bit, 1:16bit, 2:32bit etc
#define SET_DMA_CHACONFIG_DIR(x)       ((x)<<11)    // 1  0: mem=>dev, 1: dev=>mem
#define SET_DMA_CHACONFIG_MDEV(x)      ((x)<<12)    // 1  0: Fast device, 1: Memory mapped AHB device
#define SET_DMA_CHACONFIG_FIFOTHRES(x) ((x)<<13)    // 2  Fifo threshold 0:16 byte, 1:32 byte, 2:64 byte, 3:128 byte

#define GET_DMA_CHACONFIG_PORT(x)      (((x)>>0)&0x3f)
#define GET_DMA_CHACONFIG_PRIO(x)      (((x)>>6)&0x3)
#define GET_DMA_CHACONFIG_SIZE(x)      (((x)>>8)&0x7)
#define GET_DMA_CHACONFIG_DIR(x)       (((x)>>11)&1)
#define GET_DMA_CHACONFIG_MDEV(x)      (((x)>>12)&1)
#define GET_DMA_CHACONFIG_FIFOTHRES(x) (((x)>>13)&0x3)

// CHACTRL read/write control
#define SET_DMA_CHACTRL_DEVEN(x)  ((x)<<0)          // RW Write 1 to enable device
#define GET_DMA_CHACTRL_DEVEN(x)  (((x)>>0)&0x1)
#define SET_DMA_CHACTRL_BUFEN(x)  ((x)<<1)          // RW Write 1 to enable buffer
#define GET_DMA_CHACTRL_BUFEN(x)  (((x)>>1)&0x1)
#define SET_DMA_CHACTRL_RESET(x)  ((x)<<2)          // RW Write 1 to reset the channel. Read to 0 when reset is finished.
#define GET_DMA_CHACTRL_RESET(x)  (((x)>>2)&0x1)
#define SET_DMA_CHACTRL_DRAIN(x)  ((x)<<3)          // RW Write 1 to drain/fill the channel fifo. Read to 0 when drain is finished.
#define GET_DMA_CHACTRL_DRAIN(x)  (((x)>>3)&0x1)
#define SET_DMA_CHACTRL_IRQ(x)    ((x)<<4)          // RW Write 1 to ack irq. Read channel IRQ status.
#define GET_DMA_CHACTRL_IRQ(x)    (((x)>>4)&0x1)
#define SET_DMA_CHACTRL_SETIRQ(x) ((x)<<5)          //  W Write 1 to manually set IRQ.
#define SET_DMA_CHACTRL_EOF(x)    ((x)<<6)          // RW Write 1 to clear EOF condition. Read channel EOF status.
#define GET_DMA_CHACTRL_EOF(x)    (((x)>>6)&0x1)
#define SET_DMA_CHACTRL_SETEOF(x) ((x)<<7)          //  W Write 1 to manually generate EOF condition. Cleared when handled.
#define SET_DMA_CHACTRL_ADDBUF(x) ((x)<<8)          //  W Write 1 to add a descriptor to the descriptor queue.
#define SET_DMA_CHACTRL_PAUSE(x)  ((x)<<9)          //  W Write 1 to clear DEVEN and BUFEN
#define SET_DMA_CHACTRL_POLL(x)   ((x)<<10)         // RW Write 1 to set POLL condition.
#define GET_DMA_CHACTRL_POLL(x)   (((x)>>10)&0x1)
#define SET_DMA_CHACTRL_FIFOSTALL(x)((x)<<11)       // RW Write 1 to clear FIFOSTALL flag
#define GET_DMA_CHACTRL_FIFOSTALL(x)(((x)>>11)&0x1)
#define SET_DMA_CHACTRL_CLEARQUEUE(x)((x)<<12)      // RW Write 1 to clear the descriptor queue (only for debug purpose)
#define GET_DMA_CHACTRL_CLEARQUEUE(x)(((x)>>12)&0x1)

// CHASTATUS is a read-only register
#define GET_DMA_CHASTATUS_DESCPOS(x) (((x)>>0)&1)     // 1  Insertion position in the desc queue
#define GET_DMA_CHASTATUS_DESCCNT(x) (((x)>>1)&3)     // 2  Number of desc in the queue (0,1,2)
#define GET_DMA_CHASTATUS_BUFBUSY(x) (((x)>>3)&1)     // 1  Set if a buffer operation is ongoing
#define GET_DMA_CHASTATUS_DEVBUSY(x) (((x)>>4)&1)     // 1  Set if a device operation is ongoing
#define GET_DMA_CHASTATUS_FIFOFILL(x)(((x)>>8)&0x1ff) // 9  Number of bytes currently in the data fifo

// IRQ is a read-only register
#define GET_DMA_IRQ_CHANNEL(x) (((x)>>16)&0xf)      // Lowest channel signalling IRQ. 0 if no channel is signalling.
#define SET_DMA_IRQ_CHANNEL(x) ((x)<<16)

// COMMON is read/write register containing global settings
#define SET_DMA_COMMON_AHBBURSTMASK(x)  ((x)<<0)            // Allowed burst types on AHB bus {WRAP4,WRAP8,WRAP16,WRAP32,WRAP64} Normally 0x03
#define GET_DMA_COMMON_AHBBURSTMASK(x)  (((x)>>0)&0x1f)
#define SET_DMA_COMMON_XAHBBURSTMASK(x) ((x)<<5)            // Allowed burst types on XAHB bus {WRAP4,WRAP8,WRAP16,WRAP32,WRAP64} Normally 0x1f
#define GET_DMA_COMMON_XAHBBURSTMASK(x) (((x)>>5)&0x1f)

#define SET_DMA_MDEVCONFIG_SIZE(x)     ((x)<<0)     // 16  Remaining size of buffer in bytes. Only used if USESIZE is set.
#define GET_DMA_MDEVCONFIG_SIZE(x)     ((x)&0xffff)
#define SET_DMA_MDEVCONFIG_PROT(x)     ((x)<<16)    // 4   HPROT (cacheable, bufferable)
#define GET_DMA_MDEVCONFIG_PROT(x)     (((x)>>16)&0xf)
#define SET_DMA_MDEVCONFIG_BUSSEL(x)   ((x)<<20)    // 1   0: AHB, 1: XAHB
#define GET_DMA_MDEVCONFIG_BUSSEL(x)   (((x)>>20)&1)
#define SET_DMA_MDEVCONFIG_PRIO(x)     ((x)<<21)    // 1   AHB priority
#define GET_DMA_MDEVCONFIG_PRIO(x)     (((x)>>21)&1)
#define SET_DMA_MDEVCONFIG_INCRADDR(x) ((x)<<23)    // 1   0: Fixed address, 1: Increment address
#define GET_DMA_MDEVCONFIG_INCRADDR(x) (((x)>>23)&1)
#define SET_DMA_MDEVCONFIG_USESIZE(x)  ((x)<<24)    // 1   0: Unlimited size 1: Decrement size
#define GET_DMA_MDEVCONFIG_USESIZE(x)  (((x)>>24)&1)

#define SET_DMA_DESC0_BUFADDR(x)    ((x)<<0)        // 32  Physical address of buffer. Incremented
#define SET_DMA_DESC0_DEV2DEV_PORT(x) ((x)<<0)      // 5  device port select 0-31
#define SET_DMA_DESC0_DEV2DEV_SIZE(x) ((x)<<8)      // 3  device size, 0:8bit, 1:16bit, 2:32bit etc
#define SET_DMA_DESC1_BUFSIZE(x)    ((x)<<0)        // 16  Remaining size of buffer in bytes. Decremented.
#define GET_DMA_DESC1_BUFSIZE(x)    (((x)>>0)&0xffff)
#define SET_DMA_DESC1_PROT(x)       ((x)<<16)       // 4   HPROT (cacheable, bufferable)
#define SET_DMA_DESC1_BUSSEL(x)     ((x)<<20)       // 1   0: AHB, 1: XAHB
#define SET_DMA_DESC1_PRIO(x)       ((x)<<21)       // 1   AHB priority
#define SET_DMA_DESC1_IRQ(x)        ((x)<<22)       // 1   1: Generate IRQ when this descriptor has finished
#define GET_DMA_DESC1_IRQ(x)        (((x)>>22)&1)
#define SET_DMA_DESC1_EOF(x)        ((x)<<23)       // 1   1: Wait for EOF before finishing this descriptor
#define SET_DMA_DESC1_OPTION(x)     ((x)<<24)       // 2   DESC2/3 usage. 0: None. 1: Framereg. 2:Refill
#define SET_DMA_DESC1_STATE(x)      ((x)<<26)       // 2   0:START, 1:DATA, 2:STATUS, 3:DONE
#define GET_DMA_DESC1_STATE(x)      (((x)>>26)&3)
#define SET_DMA_DESC1_INITFIFO(x) ((x)<<28)         // 1   Prefetch fifo up to threshold before accepting first DREQ from device (useful in mem->dev transfer)
#define SET_DMA_DESC1_DEV2DEV(x)  ((x)<<29)         // 1   DEV2DEV descriptor. DESC0 used to store DEV2DEV fields.
#define SET_DMA_DESC2_REMAINSIZE(x) ((x)<<0)        // 32  Remaining size of the buffer in byte
#define SET_DMA_DESC3_REFILLSIZE(x) ((x)<<0)        // 16  Refill amount in byte. Unsigned 16 bit integer
#define SET_DMA_DESC3_SKIPSIZE(x)   ((x)<<16)       // 16  Skip amount in byte. Signed 16 bit integer
#define SET_DMA_DESC2_FRAMEADDR(x)  ((x)<<0)        // 32  Physical address of frame status register
#define SET_DMA_DESC3_FRAMEDATA(x)  ((x)<<0)        // 32  Data read/write from/to frame status register

#define DMA_DESC1_STATE_START   (0)
#define DMA_DESC1_STATE_DATA    (1)
#define DMA_DESC1_STATE_STATUS  (2)
#define DMA_DESC1_STATE_DONE    (3)

#define DMA_DESC1_OPTION_NONE     (0)
#define DMA_DESC1_OPTION_FRAMEREG (1)
#define DMA_DESC1_OPTION_REFILL   (2)


// SiZE constants
#define DMA_SIZE_BYTE         0
#define DMA_SIZE_SHORT        1
#define DMA_SIZE_WORD         2
#define DMA_SIZE_64BIT        3

// DIR constants
#define DMA_MEM2DEV           0
#define DMA_DEV2MEM           1

#define DMA_PORT_SMIA          0
#define DMA_PORT_AVP_BAYER_OUT 1
#define DMA_PORT_AVP_Y0        2
#define DMA_PORT_AVP_C0        3
#define DMA_PORT_AVP_Y1        4
#define DMA_PORT_AVP_C1        5
#define DMA_PORT_AVP_ANALYZE   6
#define DMA_PORT_AVP_BAYER_IN  7
#define DMA_PORT_VIDOUT_Y      8
#define DMA_PORT_VIDOUT_UV     9
#define DMA_PORT_VIDOUT_RGBA   10
#define DMA_PORT_JPEG_OUT      11
#define DMA_PORT_JPEG_Y        12
#define DMA_PORT_JPEG_UV       13
#define DMA_PORT_SDHC0         14
#define DMA_PORT_SDHC1         15
#define DMA_PORT_AES_IN        16
#define DMA_PORT_AES_OUT       17
#define DMA_PORT_USB_EP2_RD    18
#define DMA_PORT_USB_EP2_WR    19
#define DMA_PORT_USB_EP3_RD    20
#define DMA_PORT_USB_EP3_WR    21
#define DMA_PORT_UART0_RD      22
#define DMA_PORT_UART0_WR      23
#define DMA_PORT_UART1_RD      24
#define DMA_PORT_UART1_WR      25
#define DMA_PORT_SSI_IN        26
#define DMA_PORT_SSI_OUT       27
#define DMA_PORT_I2C_RX        28
#define DMA_PORT_I2C_TX        29
#define DMA_PORT_EXT0          30
#define DMA_PORT_EXT1          31 
#define DMA_PORT_ALWAYS        63 // Special always requesting port.

//-----------------------------
// EXT DMA (External DREQ)
//-----------------------------

typedef struct _tEXTDMA
{
  struct tPORT
  {
    REG32 ENABLE;
    REG32 POLARITY;  // 1 - Inverts polarity
  } PORT[2];

} tEXTDMA;

//-----------------------------
// XARB
//-----------------------------

typedef struct _tXARB
{
  REG32 AVEPRIO;    // AVE prio mode
    #define XARB_AVEPRIO_LOW        0
    #define XARB_AVEPRIO_HIGH       1
    #define XARB_AVEPRIO_ROUNDROBIN 2 // default

  REG32 PERFCTRL;
    #define SET_XARB_PERFCTRL_ENABLE(x) ((x)<<0)  // NOTE: when perfcnt is enabled, EXTCLK is always on (higher power)
    #define SET_XARB_PERFCTRL_BUS(x)    ((x)<<1)
      #define XARB_BUS_EXTMEM 0
      #define XARB_BUS_PCA    1
      #define XARB_BUS_DMA    2
      #define XARB_BUS_AVE    3
    #define SET_XARB_PERFCTRL_EVENT(x)  ((x)<<4)
      #define XARB_EVENT_NOGRANT  0   // number of no grant cycles  Unit: cycle
      #define XARB_EVENT_CYCLE    1   // number of access cycles    Unit: cycle
      #define XARB_EVENT_ACCESS   2   // number of accesses         Unit: access
      #define XARB_EVENT_WRITE    3   // number of write acceses    Unit: access
      #define XARB_EVENT_SEQ      4   // number of seq accesses     Unit: access
      #define XARB_EVENT_WAIT     5   // number of wait cycles      Unit: cycle

  REG32 PERFCNT;    // 32-bits event counter

} tXARB;

//-----------------------------
// L2-Cache Control Interface
//-----------------------------

typedef struct _tL2CACHE
{
  REG32 TYPE;         // 0x00: Cache type register
  REG32 MODE;         // 0x04: Mode
    #define SET_L2CACHE_MODE_ASSRED(x)  (((x)& 0x7)<<20) // Assocativity Reduction
    #define SET_L2CACHE_MODE_IXRED(x)   (((x)& 0x3)<<16) // Index Reduction
    #define SET_L2CACHE_MODE_IFILL(x)   (((x)& 0x1)<<12) // Instruction Cacheline Fill Enable
    #define SET_L2CACHE_MODE_LOCK(x)    (((x)& 0x1)<< 6) // Locking Mode
    #define SET_L2CACHE_MODE_BYPASS(x)  (((x)& 0x1)<< 4) // Bypass Mode
    #define SET_L2CACHE_MODE_WB(x)      (((x)& 0x1)<< 3) // Write Buffer Enable
    #define SET_L2CACHE_MODE_DFILL(x)   (((x)& 0x1)<< 2) // Data Cacheline Fill Enable

    #define GET_L2CACHE_MODE_ASSRED(x)  (((x)>>20)& 0x7) // Assocativity Reduction
    #define GET_L2CACHE_MODE_IXRED(x)   (((x)>>16)& 0x3) // Index Reduction
    #define GET_L2CACHE_MODE_IFILL(x)   (((x)>>12)& 0x1) // Instruction Cacheline Fill Enable
    #define GET_L2CACHE_MODE_LOCK(x)    (((x)>> 6)& 0x1) // Locking Mode
    #define GET_L2CACHE_MODE_BYPASS(x)  (((x)>> 4)& 0x1) // Bypass Mode
    #define GET_L2CACHE_MODE_WB(x)      (((x)>> 3)& 0x1) // Write Buffer Enable
    #define GET_L2CACHE_MODE_DFILL(x)   (((x)>> 2)& 0x1) // Data Cacheline Fill Enable

  REG32 FLUSH_ALL;    // 0x08: Flush entire cache
  REG32 FLUSH_SINGLE; // 0x0c: Flush single cache line
  REG32 CLEAN_ALL;    // 0x10: Clean entire cache
  REG32 CLEAN_SINGLE; // 0x14: Clean single cache line
  REG32 DRAIN_WB;     // 0x18: Drain write buffer
  REG32 FLUSH_STREAM; // 0x1c: Flush stream
} tL2CACHE;

#define L2CACHE_BLOCK_SIZE  32

//-----------------------------
// External Memory Interface
//-----------------------------

typedef struct _tEXTMEM
{
  #define NUM_CS (4)    // Number of chip selects

  REG32 CONFIG[NUM_CS];
      #define SET_EXTCONFIG_WIDTH(upd)   ((upd)<<0)   // 0=>byte, 1=>hword, 2=>word
      #define SET_EXTCONFIG_SETUP(upd)   ((upd)<<2)   // Setup cycles (0=>1 cycle etc)
      #define SET_EXTCONFIG_WAIT(upd)    ((upd)<<4)   // Waitstate cycles (0=>1 cycle etc)
      #define SET_EXTCONFIG_HOLD(upd)    ((upd)<<8)   // Hold cycles
      #define SET_EXTCONFIG_BWAIT(upd)   ((upd)<<10)  // burst waitstates
      #define SET_EXTCONFIG_BTYPE(upd)   ((upd)<<14)  // burst type (0=>no burst, 1=>wrap2, 2=>wrap4, 3=>wrap8, 4=>wrap16)
      #define SET_EXTCONFIG_TOFF(upd)    ((upd)<<17)  // Toff time=1+TOFF cycles
      #define SET_EXTCONFIG_WRBM(upd)    ((upd)<<19)  // 0=>normal, 1=>nBM[3:0] is write enable
      #define SET_EXTCONFIG_SLOW(upd)    ((upd)<<20)  // 0=>normal, 1=>slow mode (wait cycles *4)
      #define SET_EXTCONFIG_EXTWAIT(upd) ((upd)<<21)  // use nWAIT pin
      #define SET_EXTCONFIG_NREMODE(upd) ((upd)<<22)  // 0: normal mode, 1: nRE not active during HOLD cycles
  REG32 __pad[8-NUM_CS];
  REG32 SDCTRL1;  // Not used in Argus IV. Use SOC_DDR.SDCTRL1 etc
      #define SET_EXTSDCTRL1_ENABLE(upd) ((upd)<<0)   // Enable sdram controler on nCS4
      #define SET_EXTSDCTRL1_BSHIFT(upd) ((upd)<<1)   // Bank address position
      #define SET_EXTSDCTRL1_BWIDTH(upd) ((upd)<<5)   // Numbre of bank address bits
      #define SET_EXTSDCTRL1_RSHIFT(upd) ((upd)<<6)   // Row address position
      #define SET_EXTSDCTRL1_RWIDTH(upd) ((upd)<<9)   // Number of row address bits

  REG32 SDCTRL2;
      #define SET_EXTSDCTRL2_TRASMIN(upd) ((upd)<<0)  // tRASMIN in sdclk cycles minus one    ACTIVATE  -> PRECHARGE
      #define SET_EXTSDCTRL2_TRASMAX(upd) ((upd)<<4)  // tRASMAX in 8us units                 ACTIVATE  -> PRECHARGE
      #define SET_EXTSDCTRL2_TRRD(upd) ((upd)<<10)    // tRRD in sdclk cycles minus one       ACTIVATE  -> ACTIVATE
      #define SET_EXTSDCTRL2_TRCD(upd) ((upd)<<12)    // tRCD in sdclk cycles minus one       ACTIVATE  -> READ/WRITE
      #define SET_EXTSDCTRL2_TRP(upd) ((upd)<<14)     // tRP in sdclk cycles minus one        PRECHARGE -> any
      #define SET_EXTSDCTRL2_TRC(upd) ((upd)<<16)     // tRC in sdclk cycles minus one        REFRESH   -> any
      #define SET_EXTSDCTRL2_TRDL(upd) ((upd)<<20)    // tRDL in sdclk cycles minus one       WRITE     -> PRECHARGE
      #define SET_EXTSDCTRL2_CASL(upd) ((upd)<<22)    // CAS latency, 0=>2, 1=>3
      #define SET_EXTSDCTRL2_CLKDIV(upd) ((upd)<<23)  // SDCLK speed. HCLK/(CLKDIV+1)
      #define SET_EXTSDCTRL2_SDCLKFORCE(upd) ((upd)<<25) // Force SDCLK enable
      #define SET_EXTSDCTRL2_PCHGMODE(upd) ((upd)<<26)   // 1: Eager precharge mode
      #define SET_EXTSDCTRL2_CKEMODE(upd) ((upd)<<27)    // 1: Low-latency CKE mode

  REG32 SDRFSH;
      #define SET_EXTSDRFSH_ENABLE(upd)  ((upd)<<0)   // Refresh enable
      #define SET_EXTSDRFSH_SELF(upd)    ((upd)<<1)   // Self refresh
      #define SET_EXTSDRFSH_BURST(upd)   ((upd)<<2)   // Refresh burst length burst_len=2^BURST
      #define SET_EXTSDRFSH_PERIOD(upd)  ((upd)<<5)   // Refresh period is PERIOD+1 microsec

  REG32 SDMODE;
      #define SET_EXTSDMODE_ADDR(upd)  ((upd)<<0)     // SDRAM mode register access ADDR-pins
      #define SET_EXTSDMODE_BANK(upd)  ((upd)<<14)    // SDRAM mode register access BANK-pins

  REG32 XARB;
      #define SET_EXTXARB_ENABLE(upd)   ((upd)<<0)    // External arbitration enabled
      #define SET_EXTXARB_REQPOL(upd)   ((upd)<<1)    // Polarity of XREQ
      #define SET_EXTXARB_GRANTPOL(upd) ((upd)<<2)    // Polarity of XGRANT
      #define SET_EXTXARB_WIDTH(upd)    ((upd)<<3)    // Arbitrated data bus width (0,1,2)

  REG32 NANDCLEAR;                                    // Write-only. Clear ecc registers when written
  REG32 NANDECC0;                                     // Read-only. 24-bits smartmedia ECC for page0
  REG32 NANDECC1;                                     // Read-only. 24-bits smartmedia ECC for page1
} tEXTMEM;


typedef struct _tDDR
{
  REG32 SDCTRL1;
      #define SET_DDR_SDCTRL1_ENABLE(upd)  ((upd)<<0)   // Enable sdram controler on nCS4
      #define SET_DDR_SDCTRL1_BSHIFT(upd)  ((upd)<<1)   // Bank address position
      #define SET_DDR_SDCTRL1_BWIDTH(upd)  ((upd)<<5)   // Numbre of bank address bits
      #define SET_DDR_SDCTRL1_RSHIFT(upd)  ((upd)<<6)   // Row address position
      #define SET_DDR_SDCTRL1_RWIDTH(upd)  ((upd)<<9)   // Number of row address bits
      #define SET_DDR_SDCTRL1_DWIDTH(upd)  ((upd)<<11)  // Data bus width
      #define SET_DDR_SDCTRL1_TRIWIDTH(upd)((upd)<<13)  // Data bus tristate width (=DWIDTH)
      #define SET_DDR_SDCTRL1_DDRMODE(upd) ((upd)<<15)  // SDRAM=0, DDR=1
      #define SET_DDR_SDCTRL1_BLEN(upd)    ((upd)<<16)  // Burst length (2**BLEN). For DDR: BLEN>=1

  REG32 SDCTRL2;
      #define SET_DDR_SDCTRL2_TRASMIN(upd) ((upd)<<0)  // tRASMIN in sdclk cycles minus one    ACTIVATE  -> PRECHARGE
      #define SET_DDR_SDCTRL2_TRASMAX(upd) ((upd)<<4)  // tRASMAX in 8us units                 ACTIVATE  -> PRECHARGE
      #define SET_DDR_SDCTRL2_TRRD(upd) ((upd)<<10)    // tRRD in sdclk cycles minus one       ACTIVATE  -> ACTIVATE
      #define SET_DDR_SDCTRL2_TRCD(upd) ((upd)<<12)    // tRCD in sdclk cycles minus one       ACTIVATE  -> READ/WRITE
      #define SET_DDR_SDCTRL2_TRP(upd) ((upd)<<14)     // tRP in sdclk cycles minus one        PRECHARGE -> any
      #define SET_DDR_SDCTRL2_TRC(upd) ((upd)<<16)     // tRC in sdclk cycles minus one        REFRESH   -> any
      #define SET_DDR_SDCTRL2_TRDL(upd) ((upd)<<20)    // tRDL in sdclk cycles minus one       WRITE     -> PRECHARGE
      #define SET_DDR_SDCTRL2_CASL(upd) ((upd)<<22)    // CAS latency, 0=>2, 1=>3
      #define SET_DDR_SDCTRL2_CLKDIV(upd) ((upd)<<23)  // SDCLK speed. HCLK/(CLKDIV+1)
      #define SET_DDR_SDCTRL2_SDCLKFORCE(upd) ((upd)<<25) // Force SDCLK enable
      #define SET_DDR_SDCTRL2_PCHGMODE(upd) ((upd)<<26)   // 1: Eager precharge mode
      #define SET_DDR_SDCTRL2_CKEMODE(upd) ((upd)<<27) // 1: Low-latency CKE mode
      #define SET_DDR_SDCTRL2_TWTR(upd) ((upd)<<28)    // tWTR in sdclk cycles minus one       WRITE     -> READ
      #define SET_DDR_SDCTRL2_WTRMODE(upd) ((upd)<<30) // should be 0 (1=aggressive WTR mode, WTR individual banks)
      #define SET_DDR_SDCTRL2_THZ(upd) ((upd)<<31)     // read-->write bus turn-around time

  REG32 SDRFSH;
      #define SET_DDR_SDRFSH_ENABLE(upd)  ((upd)<<0)   // Refresh enable
      #define SET_DDR_SDRFSH_SELF(upd)    ((upd)<<1)   // Self refresh
      #define SET_DDR_SDRFSH_BURST(upd)   ((upd)<<2)   // Refresh burst length burst_len=2^BURST
      #define SET_DDR_SDRFSH_PERIOD(upd)  ((upd)<<5)   // Refresh period is PERIOD+1 microsec

  REG32 SDMODE;
      #define SET_DDR_SDMODE_ADDR(upd)  ((upd)<<0)     // SDRAM mode register access ADDR-pins
      #define SET_DDR_SDMODE_BANK(upd)  ((upd)<<14)    // SDRAM mode register access BANK-pins
      #define SET_DDR_SDMODE_NOINIT(upd)((upd)<<16)    // Skip PCHG/RFSH-init sequence (only MODE write)
      #define GET_DDR_SDMODE_BUSY(x)    (((x)>>31)&1)  // Busy writing to MODE register

  REG32 XARB;
  REG32 DLL;
} tDDR;

// SDRAM address parameters
//
// +--------+---------------------+
// |        |  BANK ADDRESS BITS  |
// | BSHIFT +----------+----------+
// |        | BWIDTH=0 | BWIDTH=1 |
// +--------+----------+----------+
// |    0   |     8    |   9:8    |
// |    1   |     9    |  10:9    |
// |    2   |     10   |  11:10   |
// |    3   |     11   |  12:11   |
// |    4   |     12   |  13:12   |
// |    5   |     20   |  21:20   |
// |    6   |     21   |  22:21   |
// |    7   |     22   |  23:22   |
// |    8   |     23   |  24:23   |
// |    9   |     24   |  25:24   |
// |    10  |     25   |  26:25   |
// |    11  |     26   |  27:26   |
// +--------+----------+----------+
//
// +--------+-------------------------------------------+
// |        |               ROW ADDRESS BITS            |
// | RSHIFT +---------------------+----------+----------+
// |        | RWIDTH=0 | RWIDTH=1 | RWIDTH=2 | RWIDTH=3 |
// +--------+----------+----------+----------+----------+
// |    0   |    18:8  |    19:8  |    20:8  |    21:8  |
// |    1   |    19:9  |    20:9  |    21:9  |    22:9  |
// |    2   |    20:10 |    21:10 |    22:10 |    23:10 |
// |    3   |    21:11 |    22:11 |    23:11 |    24:11 |
// |    4   |    22:12 |    23:12 |    24:12 |    25:12 |
// |    5   |    23:13 |    24:13 |    25:13 |    26:13 |
// |    6   |    24:14 |    25:14 |    26:14 |    27:14 |
// +--------+----------+----------+----------+----------+
//
// Formulas:
//   d - sdram databus width (8=>0, 16=>1, 32=>2)
//   c - number of column address bits of the sdram
//   b - number of bank address bits of the sdram
//   r - number of row address bits of the sdram
//
// * SDRAM size is 2^(d+c+b+r) bytes
//
// * ROW|BANK|COL|OFFSET configuration (normal)
//     SDCTRL1.DWIDTH = d
//     SDCTRL1.BSHIFT = d+c-8
//     SDCTRL1.BWDITH = b-1
//     SDCTRL1.RSHIFT = d+c+b-8
//     SDCTRL1.RWIDTH = r-11
//
// * BANK|ROW|COL|OFFSET configuration (useful in mobile sdrams, where bank shutdown is possible)
//     SDCTRL1.DWIDTH = d
//     SDCTRL1.BSHIFT = d+c+r-15
//     SDCTRL1.BWDITH = b-1
//     SDCTRL1.RSHIFT = d+c-8
//     SDCTRL1.RWIDTH = r-11
//
#define CALC_SDCTRL1(d,c,b,r) ( SET_EXTSDCTRL1_ENABLE(1) |  \
                                SET_EXTSDCTRL1_BSHIFT((d)+(c)-8) |  \
                                SET_EXTSDCTRL1_BWIDTH((b)-1) |  \
                                SET_EXTSDCTRL1_RSHIFT((d)+(c)+(b)-8) |  \
                                SET_EXTSDCTRL1_RWIDTH((r)-11))



//-----------------------------
// UART
//-----------------------------

typedef struct _tUART
{
  REG32 DATA;
  REG32 ENABLE;
    #define GET_UART_RXEN(x) (((x)>>0)&0x1)        // Receiver enable, clear fifo when disabled
    #define SET_UART_RXEN(upd) ((upd)<<0)
    #define GET_UART_TXEN(x) (((x)>>1)&0x1)        // Transmitter enable, clear fifo when disabled
    #define SET_UART_TXEN(upd) ((upd)<<1)
  REG32 CTRL;
    #define GET_UART_MODE(x) (((x)>>0)&0x1)        // 0 = RS232, 1 = IrDA
    #define SET_UART_MODE(upd) ((upd)<<0)
    #define GET_UART_AUTOFLOW(x) (((x)>>1)&0x1)    // 1 = Automatic RTS/CTS on
    #define SET_UART_AUTOFLOW(upd) ((upd)<<1)
    #define GET_UART_TXPOL(x) (((x)>>2)&0x1)       // 1 = Invert TX
    #define SET_UART_TXPOL(upd) ((upd)<<2)
    #define GET_UART_RXPOL(x) (((x)>>3)&0x1)       // 1 = Invert RX
    #define SET_UART_RXPOL(upd) ((upd)<<3)
    #define GET_UART_STOPBITS(x) (((x)>>4)&0x1)    // 0 = One stopbit, 1 = Two stopbits
    #define SET_UART_STOPBITS(upd) ((upd)<<4)
    #define GET_UART_LOOPBACK(x) (((x)>>5)&0x1)    // 1 = Loopback
    #define SET_UART_LOOPBACK(upd) ((upd)<<5)
  REG32 BAUD;                                      // Baudrate
    #define GET_UART_BAUD(x) (((x)>>0)&0x3ff)
    #define SET_UART_BAUD(upd) ((upd)<<0)
    #define UART_BAUD_2400   0x0000017f
    #define UART_BAUD_4800   0x000000bf
    #define UART_BAUD_9600   0x0000005f
    #define UART_BAUD_14400  0x0000003f
    #define UART_BAUD_28800  0x0000001f
    #define UART_BAUD_57600  0x0000000f
    #define UART_BAUD_115200 0x00000007
    #define UART_BAUD_230400 0x00000003
    #define UART_BAUD_460800 0x00000001
    #define UART_BAUD_921600 0x00000000
  REG32 IRQEN;                                      // Interrupt enable register
  REG32 STATUS;                                     // Status register
    #define GET_UART_TIMEOUT(x) (((x)>>0)&0x1)      // receive timeout
    #define SET_UART_TIMEOUT(upd) ((upd)<<0)
    #define GET_UART_RXHFULL(x) (((x)>>1)&0x1)      // 1: >= 4 bytes in receive fifo
    #define SET_UART_RXHFULL(upd) ((upd)<<1)
    #define GET_UART_RXNEMPTY(x) (((x)>>2)&0x1)     // 1: Receive fifo not empty
    #define SET_UART_RXNEMPTY(upd) ((upd)<<2)
    #define GET_UART_TXHEMPTY(x) (((x)>>3)&0x1)     // 1: <= 4 bytes in transmit fifo
    #define SET_UART_TXHEMPTY(upd) ((upd)<<3)
    #define GET_UART_TXEMPTY(x) (((x)>>4)&0x1)      // 1: Transmit fifo empty
    #define SET_UART_TXEMPTY(upd) ((upd)<<4)
    #define GET_UART_TXNFULL(x) (((x)>>5)&0x1)      // 1: Transmit fifo not full
    #define SET_UART_TXNFULL(upd) ((upd)<<5)
    #define GET_UART_TXIDLE(x) (((x)>>6)&0x1)       // 1: Uart is idle
    #define SET_UART_TXIDLE(upd) ((upd)<<6)
    #define GET_UART_OVERFLOW(x) (((x)>>7)&0x1)     // 1: Recieve fifo overflow
    #define SET_UART_OVERFLOW(upd) ((upd)<<7)
  REG32 CONFIG;                                     // Uart size register
    #define GET_UART_RXSTART(x) (((x)>>0)&0xf)      // receive window start
    #define SET_UART_RXSTART(upd) ((upd)<<0)
    #define GET_UART_RXEND(x) (((x)>>4)&0xf)        // receive window end
    #define SET_UART_RXEND(upd) ((upd)<<4)
    #define GET_UART_IRDATXLEN(x) (((x)>>8)&0x3)    // IrDA tx pulselength 00:1.67us, 01:3.33us, 10:4.83us, 11:19.50us
    #define SET_UART_IRDATXLEN(upd) ((upd)<<8)
    #define GET_UART_IRDARXLEN(x) (((x)>>10)&0xf)   // IrDA rx detect length
    #define SET_UART_IRDARXLEN(upd) ((upd)<<10)
  REG32 _PAD[(0x10000-7*4)/4];
} tUART;


//-----------------------------
// Standard Synchronous Interface
//-----------------------------

typedef struct _tSSI
{
  REG32 SSIA;
    #define GET_SSIA_TRIG(x) (((x)>>0)&0xffff)
    #define SET_SSIA_TRIG(upd) ((upd)<<0)
    #define GET_SSIA_PERIOD(x) (((x)>>16)&0xff)
    #define SET_SSIA_PERIOD(upd) ((upd)<<16)
    #define GET_SSIA_WIDTH(x) (((x)>>24)&0x1f)
    #define SET_SSIA_WIDTH(upd) ((upd)<<24)
  REG32 SSIB;
    #define GET_SSIB_TXEN(x) (((x)>>0)&0x1)
    #define SET_SSIB_TXEN(upd) ((upd)<<0)
    #define GET_SSIB_RXEN(x) (((x)>>1)&0x1)
    #define SET_SSIB_RXEN(upd) ((upd)<<1)
    #define GET_SSIB_IFRM(x) (((x)>>2)&0x1)
    #define SET_SSIB_IFRM(upd) ((upd)<<2)
    #define GET_SSIB_ICLK0(x) (((x)>>3)&0x1)
    #define SET_SSIB_ICLK0(upd) ((upd)<<3)
    #define GET_SSIB_ICLK1(x) (((x)>>4)&0x1)
    #define SET_SSIB_ICLK1(upd) ((upd)<<4)
    #define GET_SSIB_FFRM(x) (((x)>>5)&0x1)
    #define SET_SSIB_FFRM(upd) ((upd)<<5)
    #define GET_SSIB_FCLK0(x) (((x)>>6)&0x1)
    #define SET_SSIB_FCLK0(upd) ((upd)<<6)
    #define GET_SSIB_FCLK1(x) (((x)>>7)&0x1)
    #define SET_SSIB_FCLK1(upd) ((upd)<<7)
    #define GET_SSIB_DFRM(x) (((x)>>8)&0x1)
    #define SET_SSIB_DFRM(upd) ((upd)<<8)
    #define GET_SSIB_DCLK0(x) (((x)>>9)&0x1)
    #define SET_SSIB_DCLK0(upd) ((upd)<<9)
    #define GET_SSIB_DCLK1(x) (((x)>>10)&0x1)
    #define SET_SSIB_DCLK1(upd) ((upd)<<10)
    #define GET_SSIB_LBM(x) (((x)>>11)&0x1)
    #define SET_SSIB_LBM(upd) ((upd)<<11)
    #define GET_SSIB_FINV(x) (((x)>>12)&0x1)
    #define SET_SSIB_FINV(upd) ((upd)<<12)
    #define GET_SSIB_DREQ_IN(x) (((x)>>13)&0x1)
    #define SET_SSIB_DREQ_IN(upd) ((upd)<<13)
    #define GET_SSIB_DREQ_OUT(x) (((x)>>14)&0x1)
    #define SET_SSIB_DREQ_OUT(upd) ((upd)<<14)
        REG32 DATA;
}       tSSI;

//-----------------------------
// SD Host Controller
//-----------------------------
typedef struct _tSDHC
{
  REG32 SYSTEM_ADDRESS;                  // 0x00 N/A
  REG32 BLKCNT_BLKSIZE;
    #define SET_TRANSFER_BLOCK_SIZE(x) ((x)&0xfff)
    #define SET_BLOCK_COUNT(x)         ((x&0xffff)<<16)
  REG32 ARGUMENT;                        // 0x08
  REG32 CMD_TRFMOD;                      // 0x0c
    #define SET_CMD_INDEX(x)           ((x&0x3f)<<24)
    #define SET_CMD_TYPE(x)            ((x&0x3)<<22)
    #define SET_DATA_PRESENT(x)        ((x&0x1)<<21)
    #define SET_CMD_INDEX_CHECK(x)     ((x&0x1)<<20)
    #define SET_CMD_CRC_CHECK(x)       ((x&0x1)<<19)
    #define SET_RESPONSE_TYPE(x)       ((x&0x3)<<16)
    #define SET_MULTI_BLOCK(x)         ((x&0x1)<<5)
    #define SET_DATA_TRANSFER_DIR(x)   ((x&0x1)<<4)
    #define SET_AUTO_CMD12_ENABLE(x)   ((x&0x1)<<2)
    #define SET_BLOCK_COUNT_ENABLE(x)  ((x&0x1)<<1)
  REG32 RESPONSE0;                       // 0x10
  REG32 RESPONSE2;                       // 0x14
  REG32 RESPONSE4;                       // 0x18
  REG32 RESPONSE6;                       // 0x1c
  REG32 BUFFER_DATA_PORT;                // 0x20
  REG32 PRESENT_STATE;                   // 0x24
    #define GET_CMD_INHIBIT(x)           ((x>>0)&0x1)
    #define GET_DAT_INHIBIT(x)           ((x>>1)&0x1)
    #define GET_DATLA(x)                 ((x>>2)&0x1)
    #define GET_WTA(x)                   ((x>>8)&0x1)
    #define GET_RTA(x)                   ((x>>9)&0x1)
    #define GET_WBUF_ENABLE(x)           ((x>>10)&0x1)
    #define GET_RBUF_ENABLE(x)           ((x>>11)&0x1)
    #define GET_DAT_LEVEL(x)             ((x>>20)&0xf)
    #define GET_CMD_LEVEL(x)             ((x>>24)&0x1)
  REG32 WAKEUP_BLOCKGAP_POWER_HOST;      // 0x28
    #define SET_DAT_TRANSFER_WIDTH(x)        ((x&0x1)<<1)
    #define GET_DAT_TRANSFER_WIDTH(x)        ((x>>1)&0x1)
    #define SET_STOP_AT_BLOCK_GAP_REQUEST(x) ((x&0x1)<<16)
    #define GET_STOP_AT_BLOCK_GAP_REQUEST(x) ((x>>16)&0x1)
    #define SET_CONTINUE_REQUST(x)           ((x&0x1)<<17)
    #define GET_CONTINUE_REQUST(x)           ((x>>17)&0x1)
    #define SET_READ_WAIT_CONTROL(x)         ((x&0x1)<<18)
    #define GET_READ_WAIT_CONTROL(x)         ((x>>18)&0x1)
    #define SET_INTERRUPT_AT_BLOCK_GAP(x)    ((x&0x1)<<19)
    #define GET_INTERRUPT_AT_BLOCK_GAP(x)    ((x>>19)&0x1)
  REG32 SOFTWARE_TIMEOUT_CLOCK;          // 0x2c
    #define SET_SD_CLK_ENABLE(x)             ((x&0x1)<<2)
    #define GET_SD_CLK_ENABLE(x)             ((x>>2)&0x1)
    #define SET_SD_CLK_FREQ_SEL(x)           ((x&0xff)<<8)
    #define GET_SD_CLK_FREQ_SEL(x)           ((x>>8)&0xff)
    #define SET_DAT_TIMEOUT_COUNTER_VALUE(x) ((x&0xf)<<16)
    #define GET_DAT_TIMEOUT_COUNTER_VALUE(x) ((x>>16)&0xf)
    #define SET_SW_RESET_ALL(x)              ((x&0x1)<<0)
    #define SET_SW_RESET_CMD(x)              ((x&0x1)<<1)
    #define SET_SW_RESET_DAT(x)              ((x&0x1)<<2)
  REG32 INT_STATUS;                      // 0x30
    #define SET_INT_CMD_CPL(x)           ((x&0x1)<<0)
    #define GET_INT_CMD_CPL(x)           ((x>>0)&0x1)
    #define SET_INT_TRF_CPL(x)           ((x&0x1)<<1)
    #define GET_INT_TRF_CPL(x)           ((x>>1)&0x1)
    #define SET_INT_GAP_EVT(x)           ((x&0x1)<<2)
    #define GET_INT_GAP_EVT(x)           ((x>>2)&0x1)
    #define SET_INT_WBUF_READY(x)        ((x&0x1)<<4)
    #define GET_INT_WBUF_READY(x)        ((x>>4)&0x1)
    #define SET_INT_RBUF_READY(x)        ((x&0x1)<<5)
    #define GET_INT_RBUF_READY(x)        ((x>>5)&0x1)
    #define SET_INT_CARD_IRQ(x)          ((x&0x1)<<8)
    #define GET_INT_CARD_IRQ(x)          ((x>>8)&0x1)
    #define SET_INT_ERR_INTERRUPT(x)     ((x&0x1)<<15)
    #define GET_INT_ERR_INTERRUPT(x)     ((x>>15)&0x1)
    #define SET_INT_CMD_TIMEOUT_ERR(x)   ((x&0x1)<<16)
    #define GET_INT_CMD_TIMEOUT_ERR(x)   ((x>>16)&0x1)
    #define SET_INT_CMD_CRC_ERR(x)       ((x&0x1)<<17)
    #define GET_INT_CMD_CRC_ERR(x)       ((x>>17)&0x1)
    #define SET_INT_CMD_ENDBIT_ERR(x)    ((x&0x1)<<18)
    #define GET_INT_CMD_ENDBIT_ERR(x)    ((x>>18)&0x1)
    #define SET_INT_CMD_INDEX_ERR(x)     ((x&0x1)<<19)
    #define GET_INT_CMD_INDEX_ERR(x)     ((x>>19)&0x1)
    #define SET_INT_DAT_TIMEOUT_ERR(x)   ((x&0x1)<<20)
    #define GET_INT_DAT_TIMEOUT_ERR(x)   ((x>>20)&0x1)
    #define SET_INT_DAT_CRC_ERR(x)       ((x&0x1)<<21)
    #define GET_INT_DAT_CRC_ERR(x)       ((x>>21)&0x1)
    #define SET_INT_DAT_ENDBIT_ERR(x)    ((x&0x1)<<22)
    #define GET_INT_DAT_ENDBIT_ERR(x)    ((x>>22)&0x1)
    #define SET_INT_AUTO_CMD12_ERR(x)    ((x&0x1)<<24)
    #define GET_INT_AUTO_CMD12_ERR(x)    ((x>>24)&0x1)
  REG32 INT_STATUS_ENABLE;               // 0x34    // Same bit fields as INT_STATUS
  REG32 INT_SIGNAL_ENABLE;               // 0x38    // Same bit fields as INT_STATUS
  REG32 ACMD12_ERROR_STATUS;             // 0x3c
    #define SET_AUTO_CMD12_NOT_EXECUTED(x) ((x&0x1)<<0)
    #define GET_AUTO_CMD12_NOT_EXECUTED(x) ((x>>0)&0x1)
    #define SET_AUTO_CMD12_TIMEOUT_ERR(x)  ((x&0x1)<<1)
    #define GET_AUTO_CMD12_TIMEOUT_ERR(x)  ((x>>1)&0x1)
    #define SET_AUTO_CMD12_CRC_ERR(x)      ((x&0x1)<<2)
    #define GET_AUTO_CMD12_CRC_ERR(x)      ((x>>2)&0x1)
    #define SET_AUTO_CMD12_ENDBIT_ERR(x)   ((x&0x1)<<3)
    #define GET_AUTO_CMD12_ENDBIT_ERR(x)   ((x>>3)&0x1)
    #define SET_AUTO_CMD12_INDEX_ERR(x)    ((x&0x1)<<4)
    #define GET_AUTO_CMD12_INDEX_ERR(x)    ((x>>4)&0x1)
    #define SET_CMD_NOT_ISSUED_AUTO_CMD12_ERR(x) ((x&0x1)<<7)
    #define GET_CMD_NOT_ISSUED_AUTO_CMD12_ERR(x) ((x>>7)&0x1)
  REG32 CAPABILITIES;                    // 0x40
    #define GET_TIMEOUT_CLK_FREQ(x)       ((x&0x1f)>>0)
    #define GET_TIMEOUT_CLK_UNIT(x)       ((x&0x1)>>7)
    #define GET_BASE_CLK_FREQ(x)          ((x&0x1f)>>8)
    #define GET_MAX_BLOCK_LENGTH(x)       ((x&0x3)>>16)
    #define GET_HIGH_SPEED_SUPPORT(x)     ((x&0x1)>>21)
    #define GET_DMA_SUPPORT(x)            ((x&0x1)>>22)
    #define GET_SUSPEND_RESUME_SUPPORT(x) ((x&0x1)>>23)
    #define GET_VOLTAGE_SUPPORT_3v3(x)    ((x&0x1)>>24)
    #define GET_VOLTAGE_SUPPORT_3v0(x)    ((x&0x1)>>25)
    #define GET_VOLTAGE_SUPPORT_1v8(x)    ((x&0x1)>>26)
  REG32 CAPABILITIES_RESERVED;           // 0x44
  REG32 MAXCURR_CAPABILITIES;            // 0x48
    #define GET_MAX_CURR_FOR_3v3(x) ((x&0xff)>>0)
    #define GET_MAX_CURR_FOR_3v0(x) ((x&0xff)>>8)
    #define GET_MAX_CURR_FOR_1v8(x) ((x&0xff)>>16)
  REG32 MAXCURR_CAPABILITIES_RESERVED;   // 0x4c
  REG32 _PAD[39];
  REG32 DMA_ENABLE;                      // 0xec
    #define SET_DMA_ENABLE(x)            ((x&0x1)<<0)
    #define GET_DMA_ENABLE(x)            ((x>>0)&0x1)
  REG32 _PAD2[3];
  REG32 HCVER_SLOTINT_STATUS;            // 0xfc
  REG32 _PAD3[(0x10000-64*4)/4];
} tSDHC;

//-----------------------------
// Chip ID
//-----------------------------

typedef struct _tCHIPID
{
  REG32 ID;
  REG32 READY;
} tCHIPID;

//-----------------------------
// Clock Generator
//-----------------------------

typedef struct _tCLKGEN
{
  REG32 CTRL;
    #define SET_CLKGEN_LO(x) ((x)&0x3ff)
    #define GET_CLKGEN_LO(x) ((x)&0x3ff)
    #define SET_CLKGEN_HI(x) (((x)&0x3ff)<<10)
    #define GET_CLKGEN_HI(x) (((x)>>10)&0x3ff)

    #define CLKGEN_VCLK_OUT   (0)
    #define CLKGEN_PHYCLK_OUT (1)
    #define CLKGEN_CLKGEN     (2)
    #define CLKGEN_VOCLK_OUT  (3)

  REG32 _PAD[(0x10000-1*4)/4];
} tCLKGEN;

typedef struct _tETHERNET
{
  REG32 MODER;      // 0x00
    #define SET_MODER_RXEN(upd) (upd<<0)
    #define GET_MODER_RXEN(x) ((x>>0)&0x1)
    #define SET_MODER_TXEN(upd) (upd<<1)
    #define GET_MODER_TXEN(x) ((x>>1)&0x1)
    #define SET_MODER_BRO(upd) (upd<<3)
    #define GET_MODER_BRO(x) ((x>>3)&0x1)
    #define SET_MODER_PRO(upd) (upd<<5)
    #define GET_MODER_PRO(x) ((x>>5)&0x1)
    #define SET_MODER_IFG(upd) (upd<<6)
    #define GET_MODER_IFG(x) ((x>>6)&0x1)
    #define SET_MODER_LOOPBACK(upd) (upd<<7)
    #define GET_MODER_LOOPBACK(x) ((x>>7)&0x1)
    #define SET_MODER_FULLD(upd) (upd<<10)
    #define GET_MODER_FULLD(x) ((x>>10)&0x1)
    #define SET_MODER_RESET(upd) (upd<<11)
    #define GET_MODER_RESET(x) ((x>>11)&0x1)
    #define SET_MODER_CRCEN(upd) (upd<<13)
    #define GET_MODER_CRCEN(x) ((x>>13)&0x1)
    #define SET_MODER_PAD(upd) (upd<<15)
    #define GET_MODER_PAD(x) ((x>>15)&0x1)
    #define SET_MODER_RECSMALL(upd) (upd<<16)
    #define GET_MODER_RECSMALL(x) ((x>>16)&0x1)
  REG32 INT_SOURCE;     // 0x04
    #define SET_INT_SOURCE_TXB(upd)  (upd<<0)   
    #define GET_INT_SOURCE_TXB(x)     ((x>>0)&0x1) 
    #define SET_INT_SOURCE_TXE(upd)  (upd<<1)   
    #define GET_INT_SOURCE_TXE(x)     ((x>>1)&0x1) 
    #define SET_INT_SOURCE_RXF(upd)  (upd<<2)   
    #define GET_INT_SOURCE_RXF(x)     ((x>>2)&0x1) 
    #define SET_INT_SOURCE_RXE(upd)  (upd<<3)   
    #define GET_INT_SOURCE_RXE(x)     ((x>>3)&0x1) 
    #define SET_INT_SOURCE_BUSY(upd) (upd<<4)  
    #define GET_INT_SOURCE_BUSY(x)    ((x>>4)&0x1)
    #define SET_INT_SOURCE_TXC(upd)  (upd<<5)   
    #define GET_INT_SOURCE_TXC(x)     ((x>>5)&0x1) 
    #define SET_INT_SOURCE_RXC(upd)  (upd<<6)   
    #define GET_INT_SOURCE_RXC(x)     ((x>>6)&0x1) 
  REG32 INT_MASK;       // 0x08
    #define SET_INT_MASK_TXB(upd)  (upd<<0)   
    #define GET_INT_MASK_TXB(x)     ((x>>0)&0x1) 
    #define SET_INT_MASK_TXE(upd)  (upd<<1)   
    #define GET_INT_MASK_TXE(x)     ((x>>1)&0x1) 
    #define SET_INT_MASK_RXF(upd)  (upd<<2)   
    #define GET_INT_MASK_RXF(x)     ((x>>2)&0x1) 
    #define SET_INT_MASK_RXE(upd)  (upd<<3)   
    #define GET_INT_MASK_RXE(x)     ((x>>3)&0x1) 
    #define SET_INT_MASK_BUSY(upd) (upd<<4)  
    #define GET_INT_MASK_BUSY(x)    ((x>>4)&0x1)
    #define SET_INT_MASK_TXC(upd)  (upd<<5)   
    #define GET_INT_MASK_TXC(x)     ((x>>5)&0x1) 
    #define SET_INT_MASK_RXC(upd)  (upd<<6)   
    #define GET_INT_MASK_RXC(x)     ((x>>6)&0x1) 
  REG32 IPGT;           // 0x0c
  REG32 IPGR1;          // 0x10
  REG32 IPGR2;          // 0x14
  REG32 PACKETLEN;      // 0x18
  REG32 COLLCONF;       // 0x1c
  REG32 TX_BD_NUM;      // 0x20
  REG32 CTRLMODER;      // 0x24
    #define SET_CTRLMODER_PASSALL(upd) (upd<<0)
    #define GET_CTRLMODER_PASSALL(x) ((x>>0)&0x1)
    #define SET_CTRLMODER_RXFLOW(upd) (upd<<1)
    #define GET_CTRLMODER_RXFLOW(x) ((x>>1)&0x1)
    #define SET_CTRLMODER_TXFLOW(upd) (upd<<2)
    #define GET_CTRLMODER_TXFLOW(x) ((x>>2)&0x1)
  REG32 MIIMODER;       // 0x28
    #define SET_MIIMODER_CLKDIV(upd) (upd<<0)
    #define GET_MIIMODER_CLKDIV(x) ((x>>0)&0xff)
    #define SET_MIIMODER_MIINOPRE(upd) (upd<<8)
    #define GET_MIIMODER_MIINOPRE(x) ((x>>8)&0x1)
  REG32 MIICOMMAND;     // 0x2c
    #define SET_MIICOMMAND_SCANSTAT(upd) (upd<<0)
    #define GET_MIICOMMAND_SCANSTAT(x) ((x>>0)&0x1)
    #define SET_MIICOMMAND_RSTAT(upd) (upd<<1)
    #define GET_MIICOMMAND_RSTAT(x) ((x>>1)&0x1)
    #define SET_MIICOMMAND_WCTRLDATA(upd) (upd<<2)
    #define GET_MIICOMMAND_WCTRLDATA(x) ((x>>2)&0x1)
  REG32 MIIADDRESS;     // 0x30
    #define SET_MIIADDRESS_FIAD(upd) (upd<<0)
    #define GET_MIIADDRESS_FIAD(x) ((x>>0)&0x1f)
    #define SET_MIIADDRESS_RGAD(upd) (upd<<8)
    #define GET_MIIADDRESS_RGAD(x) ((x>>8)&0x1f)
  REG32 MIITX_DATA;     // 0x34
    #define SET_MIITX_DATA_CTRLDATA(upd) (upd<<0)
    #define GET_MIITX_DATA_CTRLDATA(x) ((x>>0)&0xffff)
  REG32 MIIRX_DATA;     // 0x38
    #define GET_MIIRX_DATA_PRSD(x) ((x>>0)&0xffff)
  REG32 MIISTATUS;      // 0x3c
    #define GET_MIISTATUS_LINKFAIL(x) ((x>>0)&0x1)
    #define GET_MIISTATUS_BUSY(x) ((x>>1)&0x1)
    #define GET_MIISTATUS_NVALID(x) ((x>>2)&0x1)
  REG32 MAC_ADDR0;      // 0x40
  REG32 MAC_ADDR1;      // 0x44
  REG32 ETH_HASH0_ADR;  // 0x48
  REG32 ETH_HASH1_ADR;  // 0x4c
  REG32 ETH_TXCTRL;     // 0x50

  REG32 _pad[0x100-21];

    #define ETH_TX_BD_NUM   (8)
    #define ETH_RX_BD_NUM   (8)
  REG32 TX_DESC[2*ETH_TX_BD_NUM];   // 0x400
  REG32 RX_DESC[2*ETH_RX_BD_NUM];   // 0x440
    // TX descriptor flags
    #define SET_TX_DESC_CS(upd) (upd<<0)           // Carrier sense lost
    #define GET_TX_DESC_CS(x) ((x>>0)&0x1)
    #define SET_TX_DESC_DF(upd) (upd<<1)           // Defer indication
    #define GET_TX_DESC_DF(x) ((x>>1)&0x1)
    #define SET_TX_DESC_LC(upd) (upd<<2)           // Late Collision
    #define GET_TX_DESC_LC(x) ((x>>2)&0x1)
    #define SET_TX_DESC_RL(upd) (upd<<3)           // Retransmission limit
    #define GET_TX_DESC_RL(x) ((x>>3)&0x1)
    #define SET_TX_DESC_RTRY(upd) (upd<<4)         // Retry count
    #define GET_TX_DESC_RTRY(x) ((x>>4)&0xf)
    #define SET_TX_DESC_UR(upd) (upd<<8)           // Underrun
    #define GET_TX_DESC_UR(x) ((x>>8)&0x1)
    #define SET_TX_DESC_CRC(upd) (upd<<11)
    #define GET_TX_DESC_CRC(x) ((x>>11)&0x1)
    #define SET_TX_DESC_PAD(upd) (upd<<12)
    #define GET_TX_DESC_PAD(x) ((x>>12)&0x1)
    #define SET_TX_DESC_WRAP(upd) (upd<<13)
    #define GET_TX_DESC_WRAP(x) ((x>>13)&0x1)
    #define SET_TX_DESC_IRQ(upd) (upd<<14)
    #define GET_TX_DESC_IRQ(x) ((x>>14)&0x1)
    #define SET_TX_DESC_RD(upd) (upd<<15)
    #define GET_TX_DESC_RD(x) ((x>>15)&0x1)
    #define SET_TX_DESC_STATUS(upd) (upd<<0)
    #define GET_TX_DESC_STATUS(x) ((x>>0)&0xffff)
    #define SET_TX_DESC_LEN(upd) (upd<<16)
    #define GET_TX_DESC_LEN(x) ((x>>16)&0xffff)

    // RX descriptor flags
    #define SET_RX_DESC_LC(upd) (upd<<0)            // Late collision
    #define GET_RX_DESC_LC(x) ((x>>0)&0x1)
    #define SET_RX_DESC_CRC(upd) (upd<<1)           // CRC error
    #define GET_RX_DESC_CRC(x) ((x>>1)&0x1)
    #define SET_RX_DESC_SF(upd) (upd<<2)            // Short frame
    #define GET_RX_DESC_SF(x) ((x>>2)&0x1)
    #define SET_RX_DESC_TL(upd) (upd<<3)            // Long frame
    #define GET_RX_DESC_TL(x) ((x>>3)&0x1)
    #define SET_RX_DESC_DN(upd) (upd<<4)            // Dribble nibble
    #define GET_RX_DESC_DN(x) ((x>>4)&0x1)
    #define SET_RX_DESC_IS(upd) (upd<<5)            // Invalid symbol
    #define GET_RX_DESC_IS(x) ((x>>5)&0x1)
    #define SET_RX_DESC_OR(upd) (upd<<6)            // Overrun
    #define GET_RX_DESC_OR(x) ((x>>6)&0x1)
    #define SET_RX_DESC_MISS(upd) (upd<<7)          // Frame was received because of promiscuous mode
    #define GET_RX_DESC_MISS(x) ((x>>7)&0x1)
    #define SET_RX_DESC_CF(upd) (upd<<8)            // Control frame
    #define GET_RX_DESC_CF(x) ((x>>8)&0x1)
    #define SET_RX_DESC_WRAP(upd) (upd<<13)
    #define GET_RX_DESC_WRAP(x) ((x>>13)&0x1)
    #define SET_RX_DESC_IRQ(upd) (upd<<14)
    #define GET_RX_DESC_IRQ(x) ((x>>14)&0x1)
    #define SET_RX_DESC_E(upd) (upd<<15)
    #define GET_RX_DESC_E(x) ((x>>15)&0x1)
    #define SET_RX_DESC_STATUS(upd) (upd<<0)
    #define GET_RX_DESC_STATUS(x) ((x>>0)&0xffff)
    #define SET_RX_DESC_LEN(upd) (upd<<16)
    #define GET_RX_DESC_LEN(x) ((x>>16)&0xffff)

//  // Testbench
//  REG32 DMA_CTL;       // 0x300
//    #define DMA_RX_EMPTY (1<<0)
//    #define DMA_TX_EMPTY (1<<1)
//    #define DMA_PAR_ERR  (1<<2)  // 1 to enable
//    #define CPU_BUS_REQ  (1<<3)  // 1 to enable cpu bus request
//  REG32 PHY_CRS;       // 0x304
//    #define SET_PHY_CRS_PHY_EN(upd) (upd<<0)
//    #define SET_PHY_CRS_PHY_10EN(upd) (upd<<1)
//  REG32 TXD_XO;
//    #define GET_TXD_XO(x) ((x>>0)&0xf)
//  REG32 VERSION;

} tETHERNET;

//-----------------------------
// USB slave
//-----------------------------

typedef struct _tUSB
{
  REG32 DEVICE_ADDRESS_ENABLE;
    #define SET_DEVICE_ADDRESS(x)       (x&0x7f)        // Device address 0-127
    #define GET_DEVICE_ADDRESS(x)       (x&0x7f)
    #define SET_FUNCTION_ENABLE(x)      ((x&0x1)<<7)    // Enable USB slave
    #define GET_FUNCTION_ENABLE(x)      ((x>>7)&0x1)
  REG32 ENDPOINT_ENABLE;                                // Enable generic/isochronous endpoints
    #define GET_BUSRESET_STATE(x)       (x&0x1)
    #define GET_SUSPENDED(x)            ((x>>1)&0x1)
    //#define SET_GEN_ISO_EP_ENABLE(x)    (x&0x1)
    //#define GET_GEN_ISO_EP_ENABLE(x)    (x&0x1)
  REG32 MODE_0;                                         // MODE byte 0
    #define SET_INTERRUPT_MODE(x)       ((x&0x1)<<3)    // IRQ on ERROR/NAK
    #define GET_INTERRUPT_MODE(x)       ((x>>3)&0x1)
    #define SET_ENDPOINT_MODE(x)        ((x&0x3)<<6)    // Endpoint configuration
    #define GET_ENDPOINT_MODE(x)        ((x>>6)&0x3)
  REG32 MODE_1;                                         // MODE byte 1
    #define SET_ISO_MODE(x)             ((x&0x3)<<4)    // Isochronous mode
    #define GET_ISO_MODE(x)             ((x>>4)&0x3)
    #define SET_SOF_ONLY_IRQ_MODE(x)    ((x&0x1)<<7)    // IRQ on SOF only
    #define GET_SOF_ONLY_IRQ_MODE(x)    ((x>>7)&0x1)
  REG32 SET_DMA_0;
    #define SET_DMA_EP2OUT_DEOT_ENABLE(x) ((x&0x1)<<0)
    #define GET_DMA_EP2OUT_DEOT_ENABLE(x) ((x>>0)&0x1)
    #define SET_DMA_EP2IN_DEOT_ENABLE(x)  ((x&0x1)<<1)
    #define GET_DMA_EP2IN_DEOT_ENABLE(x)  ((x>>1)&0x1)
    #define SET_DMA_EP2OUT_ENABLE(x)    ((x&0x1)<<2)
    #define GET_DMA_EP2OUT_ENABLE(x)    ((x>>2)&0x1)
    #define SET_DMA_EP2IN_ENABLE(x)     ((x&0x1)<<3)
    #define GET_DMA_EP2IN_ENABLE(x)     ((x>>3)&0x1)
    #define SET_SOF_IRQ_ENABLE(x)       ((x&0x1)<<5)
    #define GET_SOF_IRQ_ENABLE(x)       ((x>>5)&0x1)
    #define SET_EP2OUT_IRQ_ENABLE(x)    ((x&0x1)<<6)
    #define GET_EP2OUT_IRQ_ENABLE(x)    ((x>>6)&0x1)
    #define SET_EP2IN_IRQ_ENABLE(x)     ((x&0x1)<<7)
    #define GET_EP2IN_IRQ_ENABLE(x)     ((x>>7)&0x1)
  REG32 SET_DMA_1;
    #define SET_DMA_EP3OUT_DEOT_ENABLE(x) ((x&0x1)<<0)
    #define GET_DMA_EP3OUT_DEOT_ENABLE(x) ((x>>0)&0x1)
    #define SET_DMA_EP3IN_DEOT_ENABLE(x)  ((x&0x1)<<1)
    #define GET_DMA_EP3IN_DEOT_ENABLE(x)  ((x>>1)&0x1)
    #define SET_DMA_EP3OUT_ENABLE(x)    ((x&0x1)<<2)
    #define GET_DMA_EP3OUT_ENABLE(x)    ((x>>2)&0x1)
    #define SET_DMA_EP3IN_ENABLE(x)     ((x&0x1)<<3)
    #define GET_DMA_EP3IN_ENABLE(x)     ((x>>3)&0x1)
    #define SET_EP3OUT_IRQ_ENABLE(x)    ((x&0x1)<<6)
    #define GET_EP3OUT_IRQ_ENABLE(x)    ((x>>6)&0x1)
    #define SET_EP3IN_IRQ_ENABLE(x)     ((x&0x1)<<7)
    #define GET_EP3IN_IRQ_ENABLE(x)     ((x>>7)&0x1)
  REG32 INTERRUPT_STATUS_0;
    #define GET_EP0OUT_IRQ(x)           ((x>>0)&0x1)
    #define GET_EP0IN_IRQ(x)            ((x>>1)&0x1)
    #define GET_EP1OUT_IRQ(x)           ((x>>2)&0x1)
    #define GET_EP1IN_IRQ(x)            ((x>>3)&0x1)
    #define GET_EP2OUT_IRQ(x)           ((x>>4)&0x1)
    #define GET_EP2IN_IRQ(x)            ((x>>5)&0x1)
    #define GET_BUS_RESET_IRQ(x)        ((x>>6)&0x1)
    #define GET_SUSPEND_CHANGE_IRQ(x)   ((x>>7)&0x1)
  REG32 INTERRUPT_STATUS_1;
    #define GET_EP2OUT_DEOT_IRQ(x)      ((x>>0)&0x1)
    #define GET_EP2IN_DEOT_IRQ(x)       ((x>>1)&0x1)
    #define GET_EP3OUT_DEOT_IRQ(x)      ((x>>2)&0x1)
    #define GET_EP3IN_DEOT_IRQ(x)       ((x>>3)&0x1)
    #define GET_EP3OUT_IRQ(x)           ((x>>4)&0x1)
    #define GET_EP3IN_IRQ(x)            ((x>>5)&0x1)
    #define GET_SOF_IRQ(x)              ((x>>6)&0x1)

  REG32 READ_BUFFER;
  REG32 WRITE_BUFFER;
  REG32 ACK_SETUP;
  REG32 CLEAR_BUFFER;
  REG32 VALIDATE_BUFFER;
  REG32 SEND_RESUME;
  REG32 FRAME_NUMBER_LO;
  REG32 FRAME_NUMBER_HI;
    #define GET_FRAME_NUMBER_HI(x)      (x&0x7)
  REG32 DMA_EP2_OUT;
  REG32 DMA_EP2_IN;
  REG32 DMA_EP3_OUT;
  REG32 DMA_EP3_IN;
  REG32 TEST_REG;

  // Pad up to 0x007c
  //REG32 __PAD__[12];                  // offset 0x0050-0x007c
  // with TEST_REG
  REG32 __PAD__[11];                    // offset 0x0054-0x007c

  // from 0x0080 to 0x00fc
  struct ENDPOINT
  {
    REG32 SELECT_ENDPOINT;
      #define GET_FULL_EMPTY(x)         ((x>>0)&0x1)
      #define GET_STALL(x)              ((x>>1)&0x1)
    REG32 LAST_TRANSACTION_STATUS;
      #define GET_DATA_RXTX_SUCCESS(x)  ((x>>0)&0x1)
      #define GET_ERROR_CODE(x)         ((x>>1)&0xf)
      #define GET_SETUP_PACKET(x)       ((x>>5)&0x1)
      #define GET_DATA_01_PACKET(x)     ((x>>6)&0x1)
      #define GET_PREV_STAT_NOT_READ(x) ((x>>7)&0x1)
    REG32 ENDPOINT_STATUS;
      #define SET_STALL(x)              (x&0x1)
    REG32 __EP_PAD__;
  } EP[8];
} tUSB;


#define SET_AVE_BSDCTRL_BURSTMASK(x)   (((x)&0x1)<<4)
#define SET_AVE_BSDCTRL_AUTODRAIN(x)   (((x)&0x1)<<5)
#define SET_AVE_BSDCTRL_JPEGSTUFF(x)   (((x)&0x1)<<7)
#define SET_AVE_BSDCTRL_DBGSTALL(x)    (((x)&0x1)<<8)

#define GET_AVE_BSDVLC_READY(x)     (((x)>>20)&0x1)
#define GET_AVE_BSDCTRL_DIRTY(x)    (((x)>>6)&0x1)

typedef struct _tAVE
{
  // RAM0
  REG32 RAM0[3328];       // 13 kB
  
  // RAM1
  REG32 RAM1[1024];       //  4 kB

  // RAM2
  REG32 RAM2[2560];       // 10 kB

  REG32 _PAD0[1280];      //  5 kB unused

  // AVE general registers
  REG32 VERSION;
  REG32 IRQCTRL;
  REG32 _PAD1[4-2];
  REG32 STALL[4];
  
  // IME registers
  REG32 IMEINSTR;
  REG32 IMECTRL;
  REG32 IMELIMIT;
  REG32 IMEPERF;
  REG32 IMESCRIPT;
  REG32 IMEZCOST;
  REG32 _PAD2[8-6];

  // SME registers
  REG32 SMEINSTR;
  REG32 SMECTRL;
  REG32 SMELIMIT;
  REG32 SMEPERF;
  REG32 SMESCRIPT;
  REG32 SMEZCOST;
  REG32 _PAD3[8-6];

  // BP2 registers
  REG32 BPINSTR;
  REG32 BPSTATUS;
  REG32 BPCTRL;
  REG32 BPOPER;
  REG32 _PAD4[8-4];

  // MBDMA registers
  REG32 MBDCTRL;
  REG32 MBDSCRIPT;
  REG32 MBDINSTR0;
  REG32 MBDINSTR1;
  REG32 _PAD5[4];

  // BSDMA registers
  REG32 BSDADDR;
  REG32 BSDINIT;
  REG32 BSDVLC;
  REG32 BSDCTRL;
  REG32 BSDINPOS;
  REG32 BSDOUTPOS;
  REG32 _PAD6[2];

  // VLC registers
  REG32 VLCCOUNT;
  REG32 VLCRAW;
  REG32 VLCUE;
  REG32 VLCSE;
  REG32 _PAD7[16-4];
  
  // BP3 registers
  REG32 BP3INSTR;
  REG32 CHRINSTR;
  REG32 DBLINSTR;
  REG32 BP3MBMIN;
  REG32 BP3QP;
  REG32 BICINSTR;
  REG32 FILTER0;
  REG32 FILTER1;
  REG32 BP3CBP;
  REG32 DBLBSV;
  REG32 DBLBSH;
  REG32 DBLALFAY;
  REG32 DBLBETAY;
  REG32 DBLALFAC;
  REG32 DBLBETAC;
  REG32 BP3SCRIPT;
  REG32 FILTER2;
  REG32 BP3LAMBDA;
  REG32 ARBCTRL;
} tAVE;

// AVE destinations
#define SME_BUFFER_REF    	(int)(&SOC_AVE_PHYS.RAM0[0])			// AVE_42_ME 
#define SME_BUFFER_SRC    	(int)(&SOC_AVE_PHYS.RAM0[3072])		// AVE_42_ME_SRC
#define IME_BUFFER_REF    	(int)(&SOC_AVE_PHYS.RAM1[0])			// AVE_42_ME 
#define IME_BUFFER_SRC    	(int)(&SOC_AVE_PHYS.RAM1[768])		// AVE_42_ME_SRC
#define BP3_BUFFER_Y      	(int)(&SOC_AVE_PHYS.RAM2[0])			// AVE_42_Y            / AVE_81_Y
#define BP3_BUFFER_UV     	(int)(&SOC_AVE_PHYS.RAM2[512])		// AVE_42_UV, AVE_42_C / AVE_81_UV, AVE_81_C
#define BIC_BUFFER        	(int)(&SOC_AVE_PHYS.RAM2[1024])		// AVE_42_BIC
#define DBL_BUFFER_Y      	(int)(&SOC_AVE_PHYS.RAM2[1920])		// AVE_42_DBL_Y
#define DBL_BUFFER_UV     	(int)(&SOC_AVE_PHYS.RAM2[1984])		// AVE_42_DBL_UV, AVE_42_DBL_C
#define CHR_BUFFER        	(int)(&SOC_AVE_PHYS.RAM2[2048])		// AVE_42_CWIN_UV, AVE_42_CWIN_C 


//-----------------------------
// AES interface
//-----------------------------

typedef struct _tAES
{
  // RAM0
  REG32 DATA[4]; // Writing to DATA[3] starts the AES crypto in manual mode
  REG32 CTRL;
    #define SET_DECIPHER_MODE(x)     ((x&0x1)<<0)    // 1 if decipher, 0 if cipher
    #define GET_DECIPHER_MODE(x)     ((x>>0)&0x1)
    #define SET_CBC_ENABLE(x)        ((x&0x1)<<1)    // 1 if cipher block chaining enabled
    #define GET_CBC_ENABLE(x)        ((x>>1)&0x1)
    #define SET_CRYPTO_KEYSIZE(x)    ((x&0x3)<<2)    // 00=128 bit 01=192 bit 10=256 bit
    #define GET_CRYPTO_KEYSIZE(x)    ((x>>2)&0x3)
    #define SET_CRYPTO_MEMBANK(x)    ((x&0x3)<<4)    // High address bits of key
    #define GET_CRYPTO_MEMBANK(x)    ((x>>4)&0x3)
    #define SET_DMAIN_ENABLE(x)      ((x&0x1)<<6)    // 1 if input from dma
    #define GET_DMAIN_ENABLE(x)      ((x>>6)&0x1)
    #define SET_DMAOUT_ENABLE(x)     ((x&0x1)<<7)    // 1 if output to dma
    #define GET_DMAOUT_ENABLE(x)     ((x>>7)&0x1)
    #define SET_JOB_ENABLE(x)        ((x&0x1)<<8)    // 1 if job protocol parsing enables
    #define GET_JOB_ENABLE(x)        ((x>>8)&0x1)
    #define SET_AESIRQ_ENABLE(x)     ((x&0x1)<<9)    // 1 if irq enable
    #define GET_AESIRQ_ENABLE(x)     ((x>>9)&0x1)
    #define GET_CRYPTO_BUSY(x)       ((x>>31)&0x1)   // 1 if crypto running
  REG32 IRQ;
    #define GET_AESIRQ(x)            ((x) & 0x1)
    #define SET_AESIRQ_ACK(x)        ((x) & 0x1)     // 1 to ack irq
  REG32 pad0[1024-6];
  REG32 MEM[4][64];
} tAES;


//-----------------------------
// RASC controller
//-----------------------------

typedef struct _tRASC
{
  REG32 CTRL[8];                     // Single bit controlling reset of RASC. bit0 1=reset 0=run, bit1 1=pause 0=run bit2 idle out
    #define SET_RASC_RESET(x)    ((x&0x1)<<0)   // 1 if reset, 0 if run
    #define GET_RASC_RESET(x)    ((x>>0)&0x1)
    #define SET_RASC_PAUSE(x)    ((x&0x1)<<1)   // 1 if pause, 0 if run
    #define GET_RASC_PAUSE(x)    ((x>>1)&0x1)
    #define GET_RASC_IDLE(x)     ((x>>2)&0x1)   // 1: idle
  REG32 BOOTADDR[8];                 // Address for RASC to boot at
  REG32 IRQMASK[8];
  REG32 IRQSIGNAL[8];
  REG32 IRQPOLL[8];
    #define RASC_CTRL_IRQ_RASC (0)
    #define RASC_CTRL_IRQ_ARM  (1)
} tRASC;


//-----------------------------
// JPEG encoder 
//-----------------------------

typedef struct _tJPEG
{
  REG32 QUANT[2][64];
  REG32 HUFF[384];
  REG32 BASE[3];    // TODO: These are no longer used! Kept as pad.
  REG32 WIDTH;      // TODO: Obsolete. Kept as pad.
  REG32 HEIGHT;     // TODO: Obsolete. Kept as pad.
  REG32 MCU_WIDTH;  // TODO: Obsolete. Kept as pad.
  REG32 MCU_NUM;    // bit0-23
  REG32 DC_RESTART; // bit0-15
  REG32 CTRL; // bit0-2 QUANT_TAB bit3-5 HUFF_TAB bit6-7 SAMP=(XSS YSS) bit8 MONO bit9-12 XDIV bit13-18 XSKIP 
  REG32 ENABLE;     // bit0 enable (1 while jpeg working) bit1 clock on
} tJPEG;

  
//-----------------------------
// I2C controller
//-----------------------------

typedef struct _tI2C
{
  REG32 TIMING;                   // Setup registers for timing parameters
    #define SET_I2C_TIMING_HOLD(x)            ((x&0x3ff)<<0)     // sets the 'hold' part of the low scl clock phase
    #define GET_I2C_TIMING_HOLD(x)            ((x>>0)&0x3ff)
    #define SET_I2C_TIMING_SETUP(x)           ((x&0x3ff)<<10)    // sets the 'setup' part of the low scl clock phase
    #define GET_I2C_TIMING_SETUP(x)           ((x>>10)&0x3ff)
    #define SET_I2C_TIMING_HIGH(x)            ((x&0x3ff)<<20)    // sets the time for the high scl clock phase
    #define GET_I2C_TIMING_HIGH(x)            ((x>>20)&0x3ff)
  REG32 CTRL;                     // Setup register for other control
    #define SET_I2C_CTRL_FORCE_SCL(x)         ((x&0x1)<<0)       // Set this bit to control the scl by driving it high/low, clear to use high-z/low using pull-up
    #define GET_I2C_CTRL_FORCE_SCL(x)         ((x>>0)&0x1)
    #define SET_I2C_CTRL_FILTER_SCL(x)        ((x&0x3)<<1)       // Spike filter for scl line. 0=no filter, ..., 3=strong filter
    #define GET_I2C_CTRL_FILTER_SCL(x)        ((x>>1)&0x3)
    #define SET_I2C_CTRL_FILTER_SDA(x)        ((x&0x3)<<3)       // Spike filter for sda line. 0=no filter, ..., 3=strong filter
    #define GET_I2C_CTRL_FILTER_SDA(x)        ((x>>3)&0x3)
    #define SET_I2C_CTRL_SCL_LOW_EXT(x)       ((x&0x1)<<5)       // If set the slave is allowed to extend the scl low phase by driving scl low.
    #define GET_I2C_CTRL_SCL_LOW_EXT(x)       ((x>>5)&0x1)
    #define SET_I2C_CTRL_IRQ_EN_TXC(x)        ((x&0x1)<<6)       // Set to enable irq for TX data request. (Normally handled by DMA)
    #define GET_I2C_CTRL_IRQ_EN_TXC(x)        ((x>>6)&0x1)
    #define SET_I2C_CTRL_IRQ_EN_RX(x)         ((x&0x1)<<7)       // Set to enable irq for RX data request. (Normally handled by DMA)
    #define GET_I2C_CTRL_IRQ_EN_RX(x)         ((x>>7)&0x1)
    #define SET_I2C_CTRL_IRQ_EN_NOACK(x)      ((x&0x1)<<8)       // Set to enable irq when entering noack state
    #define GET_I2C_CTRL_IRQ_EN_NOACK(x)      ((x>>8)&0x1)
  REG32 STATUS;                   // Read only register for current status
    #define SET_I2C_STATUS_IRQ_CMD(x)         ((x&0x1)<<0)       // Set when the irq command is received. Write 1 to clear.
    #define GET_I2C_STATUS_IRQ_CMD(x)         ((x>>0)&0x1)
    #define SET_I2C_STATUS_NOACK(x)           ((x&0x1)<<1)       // Set when the noack state is reached. Write 1 to clear.
    #define GET_I2C_STATUS_NOACK(x)           ((x>>1)&0x1)
    #define SET_I2C_STATUS_DREQ_TXC(x)        ((x&0x1)<<2)       // High when block is ready to receive data on TXC register.
    #define GET_I2C_STATUS_DREQ_TXC(x)        ((x>>2)&0x1)
    #define SET_I2C_STATUS_DREQ_RX(x)         ((x&0x1)<<3)       // High when block has new data on RX register.
    #define GET_I2C_STATUS_DREQ_RX(x)         ((x>>3)&0x1)
    #define SET_I2C_STATUS_STATE(x)           ((x&0xf)<<4)       // Internal state, for debug only.
    #define GET_I2C_STATUS_STATE(x)           ((x>>4)&0xf)
    #define SET_I2C_STATUS_JOB_CNT(x)         ((x&0x1f)<<8)      // Internal counter, for debug only.
    #define GET_I2C_STATUS_JOB_CNT(x)         ((x>>8)&0x1f)
  REG32 DUMMY;                    // Not used
  REG32 TXC;                      //
  REG32 RX;                       //
} tI2C;

typedef struct _tMBDMA
{
  REG32 IADDR;
  REG32 IXOFFS;
  REG32 BUSATTRLO;
  REG32 BUSATTRHI;
  REG32 ICNT;
  REG32 STATUS;
} tMBDMA;

//-----------------------------
// SMIA 
//-----------------------------

typedef struct _tSMIA
{
  REG32 CTRL;         
    #define SET_SMIA_CTRL_CLK_EN(x)            ((x&0x1)<<0)
    #define GET_SMIA_CTRL_CLK_EN(x)            ((x>>0)&0x1)    
    #define SET_SMIA_CTRL_STROBED(x)           ((x&0x1)<<1)
    #define GET_SMIA_CTRL_STROBED(x)           ((x>>1)&0x1)
    #define SET_SMIA_CTRL_AVP_EN(x)            ((x&0x1)<<2)
    #define GET_SMIA_CTRL_AVP_EN(x)            ((x>>2)&0x1)
    #define SET_SMIA_CTRL_PAD_END(x)           ((x&0x1)<<3)
    #define GET_SMIA_CTRL_PAD_END(x)           ((x>>3)&0x1)        
    #define SET_SMIA_CTRL_PAD_PDD(x)           ((x&0x1)<<4)
    #define GET_SMIA_CTRL_PAD_PDD(x)           ((x>>4)&0x1)        
    #define SET_SMIA_CTRL_PAD_ENC(x)           ((x&0x1)<<5)
    #define GET_SMIA_CTRL_PAD_ENC(x)           ((x>>5)&0x1)        
    #define SET_SMIA_CTRL_PAD_PDC(x)           ((x&0x1)<<6)
    #define GET_SMIA_CTRL_PAD_PDC(x)           ((x>>6)&0x1)        
    #define SET_SMIA_CTRL_TUNE(x)              ((x&0xF)<<7)
    #define GET_SMIA_CTRL_TUNE(x)              ((x>>7)&0xF)    
    #define SET_SMIA_CTRL_DIVRES(x)            ((x&0x1)<<11)
    #define GET_SMIA_CTRL_DIVRES(x)            ((x>>11)&0x1)
    #define SET_SMIA_CTRL_RESET(x)             ((x&0x1)<<12)
    #define GET_SMIA_CTRL_RESET(x)             ((x>>12)&0x1)

  REG32 VCH;     
    #define SET_SMIA_VCH_AVP_CH(x)             ((x&0x7)<<0)
    #define GET_SMIA_VCH_AVP_CH(x)             ((x>>0)&0x7)    
    #define SET_SMIA_VCH_START_LINE(x)         ((x&0xFFF)<<3)
    #define GET_SMIA_VCH_START_LINE(x)         ((x>>3)&0xFFF)    
    #define SET_SMIA_VCH_END_LINE(x)           ((x&0xFFF)<<15)
    #define GET_SMIA_VCH_END_LINE(x)           ((x>>15)&0xFFF)        
    #define SET_SMIA_VCH_RAW_MODE(x)           ((x&0x3)<<27)
    #define GET_SMIA_VCH_RAW_MODE(x)           ((x>>27)&0x3)        
    #define SET_SMIA_VCH_DBG_MODE(x)           ((x&0x1)<<29)
    #define GET_SMIA_VCH_DBG_MODE(x)           ((x>>29)&0x1)    
    #define SET_SMIA_VCH_ENABLE_DPCM(x)        ((x&0x1)<<30)
    #define GET_SMIA_VCH_ENABLE_DPCM(x)        ((x>>30)&0x1)    
  
  REG32 STATUS;
    #define GET_SMIA_STATUS_CRC(x)             ((x>>0)&0xFFF)
    #define GET_SMIA_STATUS_ERR_DMA(x)         ((x>>12)&0x1)
    #define SET_SMIA_STATUS_ERR_DMA(x)         ((x&0x1)<<12)
    #define GET_SMIA_STATUS_ERR_AVP(x)         ((x>>13)&0x1)
    #define SET_SMIA_STATUS_ERR_AVP(x)         ((x&0x1)<<13)
    #define GET_SMIA_STATUS_IRQ(x)             ((x>>14)&0x1)
    #define SET_SMIA_STATUS_IRQ(x)             ((x&0x1)<<14)

} tSMIA;

//-----------------------------
// Host Interface
//-----------------------------

typedef struct _tHOSTIF
{
  REG32 CTRL;                                                  // Accessable by Slave
    #define SET_HOSTIF_CTRL_EN(x)          ((x&0x1)<<0)        // Enable host IF
    #define GET_HOSTIF_CTRL_EN(x)          ((x>>0)&0x1)
    #define SET_HOSTIF_CTRL_LOOPBACK(x)    ((x&0x1)<<1)        // Enable Extmem -> host if loopback
    #define GET_HOSTIF_CTRL_LOOPBACK(x)    ((x>>1)&0x1)
    #define SET_HOSTIF_CTRL_DRIVE_NWAIT(x) ((x&0x1)<<2)        // Drive nWAIT (otherwise tristare on 1)
    #define GET_HOSTIF_CTRL_DRIVE_NWAIT(x) ((x>>2)&0x2)
    #define SET_HOSTIF_CTRL_DRIVE_IRQ(x)   ((x&0x1)<<3)        // Drive IRQ (otherwise tristare on 1)
    #define GET_HOSTIF_CTRL_DRIVE_IRQ(x)   ((x>>3)&0x2)
    #define SET_HOSTIF_CTRL_ID(x)          ((x&0x7)<<4)        // 3 bit slave ID
    #define GET_HOSTIF_CTRL_ID(x)          ((x>>4)&0x7)
    #define SET_HOSTIF_CTRL_TIMEOUT(x)     ((x&0x3)<<7)        // 2 bit timeout
    #define GET_HOSTIF_CTRL_TIMEOUT(x)     ((x>>7)&0x3)
    #define SET_HOSTIF_CTRL_WAIT_EN(x)     ((x&0x1)<<9)        // 2 bit timeout
    #define GET_HOSTIF_CTRL_WAIT_EN(x)     ((x>>9)&0x1)

  REG32 CMDIRQ_HOST;                                           // Accessable by slave and host
    #define SET_HOSTIF_CMDIRQ_HOST_IRQ(x) ((x&0x1)<<0)         // HOST_IRQ
    #define GET_HOSTIF_CMDIRQ_HOST_IRQ(x) ((x>>0)&0x1)
    #define SET_HOSTIF_CMDIRQ_HOST_CMD(x) ((x&0xf7)<<1)        // HOST_CMD 7 bits
    #define GET_HOSTIF_CMDIRQ_HOST_CMD(x) ((x>>1)&0xf7)

  REG32 CMDIRQ_SLAVE;                                          // Accessable by slave and host
    #define SET_HOSTIF_CMDIRQ_SLAVE_IRQ(x) ((x&0x1)<<0)        // SLAVE_IRQ
    #define GET_HOSTIF_CMDIRQ_SLAVE_IRQ(x) ((x>>0)&0x1)
    #define SET_HOSTIF_CMDIRQ_SLAVE_CMD(x) ((x&0xf7)<<1)       // SLAVE_CMD 7 bits
    #define GET_HOSTIF_CMDIRQ_SLAVE_CMD(x) ((x>>1)&0xf7)

  REG32 IRQ_MASK;                                              // Accessable by slave
    #define SET_HOSTIF_IRQ_MASK_HOST(x)    ((x&0x1)<<0)        // Mask HOST_IRQ (to slave)
    #define GET_HOSTIF_IRQ_MASK_HOST(x)    ((x>>0)&0x1)
    #define SET_HOSTIF_IRQ_MASK_SLAVE(x)   ((x&0x1)<<1)        // Mask SLAVE_IRQ (to slave)
    #define GET_HOSTIF_IRQ_MASK_SLAVE(x)   ((x>>1)&0x1)

  REG32 HOST_ADDR;                                             // Accessable by slave and host
    #define SET_HOSTIF_HOST_ADDR(x)        ((x&0xffffffff)<<0) // 32 bit Host address
    #define GET_HOSTIF_HOST_ADDR(x)        ((x>>0)&0xffffff)   // HA3 & HA2 & HA1 & HA0

  REG32 pad0[256-5];

  // RAM 1 kByte
  REG32 RAM[256];                                              // Slave and host

} tHOSTIF;

//-----------------------------
// FPGA specific registers
//-----------------------------

typedef struct _tFPGA
{
  REG32 VERSION;
  REG32 MAGIC;
} tFPGA;

//-----------------------------
// GPIO pin numbers
//-----------------------------

#define GPIO_ADDR18         0  // ALT_FUNC
#define GPIO_ADDR19         1  // ALT_FUNC
#define GPIO_ADDR20         2  // IN_PU, HOSTIF_BOOT_ID0
#define GPIO_ADDR21         3  // IN_PU, HOSTIF_BOOT_ID1
#define GPIO_ADDR22         4  // IN_PU, HOSTIF_BOOT_ID2
#define GPIO_ADDR23         5  // IN_PU
#define GPIO_ADDR24         6  // IN_PU
#define GPIO_nCS0           7  // ALT_FUNC
#define GPIO_nCS1           8  // ALT_FUNC
#define GPIO_nCS2           9  // IN_PU
#define GPIO_nCS3           10 // ALT_FUNC
#define GPIO_NAND_nRE       11 // IN_PU
#define GPIO_NAND_nWE       12 // IN_PU
#define GPIO_nWAIT          13 // IN_PU
#define GPIO_XREQ           14 // IN_PU
#define GPIO_XGRANT         15 // IN_PU
#define GPIO_DREQ0          16 // IN_PU
#define GPIO_DREQ1          17 // IN_PU
#define GPIO_PWM0           18 // IN_PU
#define GPIO_PWM1           19 // IN_PU
#define GPIO_SSI_RX         20 // IN_PU
#define GPIO_SSI_TX         21 // IN_PU
#define GPIO_SSI_FRM        22 // IN_PU
#define GPIO_SSI_CLK        23 // IN_PU
#define GPIO_nRESET_OUT     24 // ALT_FUNC
#define GPIO_UART0_RXD      25 // IN_PU
#define GPIO_UART0_TXD      26 // IN_PU
#define GPIO_UART0_RTS      27 // IN_PU
#define GPIO_UART0_CTS      28 // IN_PU
#define GPIO_UART1_RXD      29 // IN_PU
#define GPIO_UART1_TXD      30 // IN_PU
#define GPIO_CLKGEN         31 // IN_PU
#define GPIO_LED            32 // IN_PU
#define GPIO_ETH_MDC        33 // IN_PU  -- TODO: Check reset state
#define GPIO_ETH_MDIO       34 // IN_PU
#define GPIO_ETH_TXD0       35 // IN_PU
#define GPIO_ETH_TXD1       36 // IN_PU
#define GPIO_ETH_TXD2       37 // IN_PU
#define GPIO_ETH_TXD3       38 // IN_PU
#define GPIO_ETH_COL        39 // IN_PU
#define GPIO_ETH_CRS        40 // IN_PU
#define GPIO_ETH_TXEN       41 // IN_PU
#define GPIO_ETH_TXER       42 // IN_PU
#define GPIO_ETH_TXCLK      43 // IN_PU
#define GPIO_ETH_RXD0       44 // IN_PU
#define GPIO_ETH_RXD1       45 // IN_PU
#define GPIO_ETH_RXD2       46 // IN_PU
#define GPIO_ETH_RXD3       47 // IN_PU
#define GPIO_ETH_RXDV       48 // IN_PU
#define GPIO_ETH_RXER       49 // IN_PU
#define GPIO_ETH_RXCLK      50 // IN_PU
#define GPIO_ETH_PHYCLK     51 // IN_PU
#define GPIO_VOCLK          52 // IN_PU
#define GPIO_VOHSYNC        53 // IN_PU
#define GPIO_VOVSYNC        54 // IN_PU
#define GPIO_VODATA0        55 // IN_PU
#define GPIO_VODATA1        56 // IN_PU
#define GPIO_VODATA2        57 // IN_PU
#define GPIO_VODATA3        58 // IN_PU
#define GPIO_VODATA4        59 // IN_PU
#define GPIO_VODATA5        60 // IN_PU
#define GPIO_VODATA6        61 // IN_PU
#define GPIO_VODATA7        62 // IN_PU
#define GPIO_VAD0           63 // IN_PU
#define GPIO_VAD1           64 // IN_PU
#define GPIO_VAD2           65 // IN_PU
#define GPIO_VAD3           66 // IN_PU
#define GPIO_VAD4           67 // IN_PU
#define GPIO_VAD5           68 // IN_PU
#define GPIO_VAD6           69 // IN_PU
#define GPIO_VAD7           70 // IN_PU
#define GPIO_VAD8           71 // IN_PU
#define GPIO_VAD9           72 // IN_PU
#define GPIO_VAD10          73 // IN_PU
#define GPIO_VAD11          74 // IN_PU
#define GPIO_VFST           75 // IN_PU
#define GPIO_VCLK           76 // IN_PU
#define GPIO_VCLK_OUT       77 // IN_PU
#define GPIO_VPIX           78 // IN_PU
#define GPIO_VIDEO0         79 // IN_PU
#define GPIO_VIDEO1         80 // IN_PU
#define GPIO_VIDEO2         81 // IN_PU
#define GPIO_VIDEO3         82 // IN_PU
#define GPIO_SDIO0_DATA0    83 // IN_PU
#define GPIO_SDIO0_DATA1    84 // IN_PU
#define GPIO_SDIO0_DATA2    85 // IN_PU
#define GPIO_SDIO0_DATA3    86 // IN_PU
#define GPIO_SDIO0_CMD      87 // IN_PU
#define GPIO_SDIO0_CLK      88 // IN_PU
#define GPIO_SDIO1_DATA0    89 // IN_PU
#define GPIO_SDIO1_DATA1    90 // IN_PU
#define GPIO_SDIO1_DATA2    91 // IN_PU
#define GPIO_SDIO1_DATA3    92 // IN_PU
#define GPIO_SDIO1_CMD      93 // IN_PU
#define GPIO_SDIO1_CLK      94 // IN_PU
#define GPIO_SDIO1_DATA_DIR 95 // IN_PU
#define GPIO_SDIO1_CMD_DIR  96 // IN_PU


#endif // ARGUS4_H
