#ifndef __ARGUS_ADDR_H__
#define __ARGUS_ADDR_H__

/* Physical map */

#define BOOT_ROM_BASE_P     0x00000000    /* 4 kB */
#define OCRAM_BASE_P        0x10000000    /* 32 or 64kB */
#define APB_BASE_P          0x20000000    /* 2 Mb */
#define ZEROS_BASE_P        0x30000000    /* 256 Mb */
#define L2CACHE_CTRL_BASE_P 0x40000000    /* 4 kB */
#ifdef CONFIG_ARGUS_3
#define L2CACHE_STAT_BASE_P 0x50000000    /* 128 kB in Argus2, 24 kB in Argus3 */
#else
// Argus-4
#define L2CACHE_STAT_BASE_P (OCRAM_BASE_P) 
#define ETH_BASE_P          0x48000000
#define AES_BASE_P          0x50000000
#endif
#define DMA_BASE_P          0x60000000    /* 8 kB */
#define VSYS_BASE_P         0x70000000    // 64 kB. Argus-4 only.
#define EXT0_BASE_P         0x80000000    /* 256 Mb */
#define EXT1_BASE_P         0x90000000    /* 256 Mb */
#define EXT2_BASE_P         0xa0000000    /* 256 Mb */
#define EXT3_BASE_P         0xb0000000    /* 256 Mb */
#define EXT4_BASE_P         0xc0000000    /* 256 Mb */
#define EXT5_BASE_P         0xd0000000    // 256 Mb. Argus-3 only.

/* Virtual map (not all things are mapped in reality -
 * see arch/arm/mach-argus/mm.c). The sizes refer to the map
 * sizes, not the actual underlying accessible memory size.
 */

#ifndef PHYSICAL_ADDRESSING
#define EXT0_NOCACHE_V        0xe0000000    /* 32 Mb */
#define EXT0_CACHE_V          0xe2000000    /* 32 Mb */
#define APB_NOCACHE           0xe4000000    /* 2 Mb */
#define OCRAM_BASE_V          0xe4200000    /* 1 Mb */
#define L2CACHE_CTRL_NOCACHE  0xe4300000    /* 1 Mb */
#ifdef CONFIG_ARGUS_3
#define L2CACHE_STAT_BASE_V   0xe4400000    /* 1 Mb */
#else
// On Argus-4, OCRAM_BASE and L2CACHE_STAT_BASE are the same
// so we don't want to map the same thing twice
#define L2CACHE_STAT_BASE_V   OCRAM_BASE_V
#endif
#define EXT2_NOCACHE_V        0xe4500000    /* 1 Mb */
#define EXT5_NOCACHE1_V       0xe4600000    /* 1 Mb */
#define EXT5_NOCACHE2_V       0xe4700000    /* 1 Mb */
#define EXT5_NOCACHE3_V       0xe4800000    /* 1 Mb */
#define DMA_NOCACHE           0xe4900000    /* 1 Mb */
#define NAND_NOCACHE_DATA_V   0xe4a00000    /* 1 Mb */
#define VSYS_NOCACHE          0xe4b00000    /* 1 Mb */
#ifdef CONFIG_ARGUS_4
#define ETH_NOCACHE           0xe4c00000    /* 1 Mb */
#define AES_NOCACHE           0xe4d00000    /* 1 Mb */
#endif
#define EXT4_NOCACHE_V        0xe6000000    /* 128 Mb */
/*#define ZEROS_BASE_V        0x30000000*/  /* 256 Mb */
#else
#define EXT0_NOCACHE_V        EXT0_BASE_P            /* 32 Mb */
#define APB_NOCACHE           APB_BASE_P             /* 2 Mb */
#define OCRAM_BASE_V          OCRAM_BASE_P           /* 1 Mb */
#define L2CACHE_CTRL_NOCACHE  L2CACHE_CTRL_BASE_P    /* 1 Mb */
#define L2CACHE_STAT_BASE_V   L2CACHE_STAT_BASE_P    /* 1 Mb */
#endif
#endif /* __ARGUS_ADDR_H__ */
