/*
 * The device /dev/cryptocop is accessible using this driver using
 * CRYPTO_MAJOR (254) and minor number 0.
 */

#ifndef CRYPTO_H
#define CRYPTO_H

#define OP_ENCRYPT 0
#define OP_DECRYPT 1

#define CRYPTO_MAX_KEY_LENGTH      (32)
#define CRYPTO_MAX_IV_LENGTH       (16)

typedef unsigned long long int crypto_session_id;

typedef enum {
	crypto_cipher_mode_ecb = 0,
	crypto_cipher_mode_cbc,
} crypto_cipher_mode;

struct crypto_session_op
{
	crypto_session_id    ses_id;
	crypto_cipher_mode   cmode; /* ECB, CBC */
	unsigned char        *key;
	size_t               keylen;
};

struct crypto_process_op
{
	crypto_session_id    ses_id;

	/* Indata. */
        unsigned char        *indata;
	size_t               inlen; /* Total indata length. */
	unsigned char        cipher_iv[CRYPTO_MAX_IV_LENGTH];

	/* Configuration */
	unsigned char        op;

	/* Outdata. */
	unsigned char        *outdata;
	size_t               outlen;
};

#define ARTPECBCRYPTO_IOCTYPE         (250)

#define CRYPTO_IO_CREATE_SESSION  _IOWR(ARTPECBCRYPTO_IOCTYPE, 1, struct crypto_session_op)
#define CRYPTO_IO_CLOSE_SESSION   _IOW(ARTPECBCRYPTO_IOCTYPE, 2, struct crypto_session_op)
#define CRYPTO_IO_PROCESS_OP      _IOWR(ARTPECBCRYPTO_IOCTYPE, 3, struct crypto_process_op)

#define CRYPTO_IO_MAXNR             (3)

#endif
