#ifndef SSI_H
#define SSI_H

// ioctl commands

// Set the Argus SSIA register which controls speed and datasize (see argus3.h)
// The appropriate DMA transfer size has to be set separately.
#define SSI_SET_SSIA 0
// Set the Argus SSIB register which controls the format (see argus3.h)
// NOTE: TX/RX enable bits doesn't have to be set here - they are enabled when
// buffers are added as appropriate.
#define SSI_SET_SSIB 1
// Request a buffer input (blocking if nothing available)
#define SSI_INPUT 2
// Request a buffer output (blocking if full)
#define SSI_OUTPUT 3
// Set the DMA transfer size in bytes (1, 2 or 4 is supported)
// For stereo codecs, it is possible to select a single channel from the stereo
// pair using a trick here. The codec sends 16 bits left and then 16 bits right.
// If the SSI word size is set to 16 bits and the DMA size is set to 2 bytes, 
// port will read and store the L channel. If the SSI size is 32 bits and the
// DMA size stays at 2 bytes though, the R channel will be stored since the DMA
// will write the LSB's of the SSI transfer longword.
#define SSI_SET_DMA_WIDTH 4

// Select the read-mode
#define SSI_SET_INPUT_MODE 5

enum {
	// When there are no readers, halt the interface and flush any
	// read buffers (default)
	SSI_INPUT_MODE_HALT = 0,
	// Keep the input running all the time, starting with the moment
	// this mode was activated.
	//
	// If all buffers are filled before any user reads them, they
	// are flushed and a flag is set which keeps *one* buffer active
	// in the queue, so that a subsequent read will get a buffer
	// directly, and then the flag is cleared again and all buffers
	// are allowed to be buffered (until the next time the user
	// lags)

	SSI_INPUT_MODE_CONTINUOUS = 1
};

// Request struct to pass through the ioctl interface to read data with timestamps
// and write a buffer. All buffers have the same constant size.

#define SSI_BUFFER_SIZE 2048

struct ssi_request {
	char *buf;
	struct timeval timestamp;
};

#endif

