/*
 * linux/include/asm-cris/arch-v32/board.h
 *
 * Types for board-specific data.
 *
 * Copyright (C) 2007  Axis Communications AB
 */

#ifndef __ARCH_V32_BOARD_H
#define __ARCH_V32_BOARD_H

/* A tuple for a regi base and an interrupt.  */
struct crisv32_regi_n_int {
	u32 regi;
	u32 irq;
};

/*
 * SPI and SD/MMC types, arranged such that the data for mmc_spi is an
 * add-on to SPI; the SPI data does not contain anything about SD/MMC
 * and the SPI controller driver can't access it.  SPI data is
 * arranged to allow sharing common functions between SSER and GPIO.
 */

/* Hardware identification for the bitbanged GPIO SPI controller.  */
struct crisv32_spi_gpio_controller_data {
	/* Names of the pins.  */
	const char *cs;
	const char *miso;
	const char *mosi;
	const char *sclk;
};

/* Similar for the SSER SPI controller.  */
struct crisv32_spi_sser_controller_data {
	/* How to connect pins and claim the SSER interface and the DMA
	   channels.  */
	int (*iface_allocate)(struct crisv32_regi_n_int *sser,
			      struct crisv32_regi_n_int *dmain,
			      struct crisv32_regi_n_int *dmaout);
	void (*iface_free)(void);

	/* Whether DMA is to be used.  */
	int using_dma;
};

struct crisv32_mmc_spi_pinstate;

/*
 * Regardless of SPI controller, these pins are needed in addition to
 * the SPI pins when the used as a SD/MMC SPI controller.
 */
struct crisv32_mmc_spi_pindata {
	/* Names of the pins.  */
	const char *card_detect;
	const char *write_protect;

	/* Related private state to interface to the mmc_spi API. */
	struct crisv32_mmc_spi_pinstate *pinstate;
};

/* When SD/MMC SPI on GPIO, here's all the hardware-identifying data.  */
struct crisv32_mmc_spi_gpio_hwdata {
	struct crisv32_spi_gpio_controller_data spi;
	struct crisv32_mmc_spi_pindata mmc;
};

/* Similar for SSER.  */
struct crisv32_mmc_spi_sser_hwdata {
	struct crisv32_spi_sser_controller_data spi;
	struct crisv32_mmc_spi_pindata mmc;
};

#endif /* __ARCH_V32_BOARD_H */
